package net.sf.doolin.gui.action;

import net.sf.doolin.gui.expression.GUIExpression;

import org.springframework.beans.factory.annotation.Required;

import com.google.common.base.Function;

/**
 * This action delegates the execution to another action, adapting the action
 * context to a new value.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class DelegateGUIAction extends AbstractGUIAction {

	private GUIAction action;
	private Function<ActionContext, ActionContext> actionContextConverter;

	@Override
	protected void doExecute(ActionContext context) throws Exception {
		// Converts the action context
		ActionContext delegateContext = getDelegateContext(context);
		// Delegates
		this.action.execute(delegateContext);
	}

	/**
	 * Returns the target action
	 * 
	 * @return Action
	 */
	public GUIAction getAction() {
		return this.action;
	}

	/**
	 * Returns the action context converter
	 * 
	 * @return Action context converter
	 */
	public Function<ActionContext, ActionContext> getActionContextConverter() {
		return this.actionContextConverter;
	}

	/**
	 * Gets the delegated action context
	 * 
	 * @param context
	 *            Current action context
	 * @return Action context to use for the delegate
	 */
	protected ActionContext getDelegateContext(ActionContext context) {
		ActionContext delegateContext = this.actionContextConverter
				.apply(context);
		return delegateContext;
	}

	@Override
	public GUIExpression getIconIDExpression(ActionContext actionContext) {
		return this.action
				.getIconIDExpression(getDelegateContext(actionContext));
	}

	@Override
	public String getId() {
		return this.action.getId();
	}

	@Override
	public GUIExpression getShortcutExpression(ActionContext actionContext) {
		return this.action
				.getShortcutExpression(getDelegateContext(actionContext));
	}

	@Override
	public GUIExpression getTextExpression(ActionContext actionContext) {
		return this.action.getTextExpression(getDelegateContext(actionContext));
	}

	@Override
	public GUIExpression getTipExpression(ActionContext actionContext) {
		return this.action.getTipExpression(getDelegateContext(actionContext));
	}

	/**
	 * Sets the target action
	 * 
	 * @param action
	 *            Action
	 */
	@Required
	public void setAction(GUIAction action) {
		this.action = action;
	}

	/**
	 * Sets the action context converter
	 * 
	 * @param actionContextConverter
	 *            Action context converter
	 */
	public void setActionContextConverter(
			Function<ActionContext, ActionContext> actionContextConverter) {
		this.actionContextConverter = actionContextConverter;
	}

}
