package net.sf.doolin.gui.action;

import net.sf.doolin.gui.action.support.DefaultActionContextDataExtractor;
import net.sf.doolin.gui.action.task.GUITask;
import net.sf.doolin.gui.action.task.GUITaskExecutor;
import net.sf.doolin.gui.expression.GUIExpression;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Required;

/**
 * This action launches a {@link GUITask} using an associated
 * {@link GUITaskExecutor}.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of the input to the {@link GUITask}
 * @param <R>
 *            Type of the return from the {@link GUITask}
 */
public class GUITaskAction<B, R> extends AbstractSimpleGUIAction {

	private GUITask<B, R> task;
	private GUITaskExecutor<B, R> taskExecutor;
	private String prompt;

	@Override
	@SuppressWarnings("unchecked")
	protected void doExecute(ActionContext context) throws Exception {
		// Bean
		B bean = (B) getActionContextData(context);
		// Prompt
		if (StringUtils.isNotBlank(this.prompt)) {
			String promptMessage = new GUIExpression(context
					.getSubscriberValidator(), bean, this.prompt).getValue();
			boolean ok = context.getApplication().getInfoService().confirm(
					context, "GUITaskAction.prompt", promptMessage);
			if (!ok) {
				return;
			}
		}
		// Task execution
		this.taskExecutor.execute(context, bean, this.task);
	}

	/**
	 * Gets the expression that is used to prompt the user before the task is
	 * executed. If this expression is not defined, no prompt is displayed to
	 * the user.
	 * 
	 * @return Expression or <code>null</code> to disable the prompt.
	 */
	public String getPrompt() {
		return this.prompt;
	}

	/**
	 * Sets the expression that is used to prompt the user before the task is
	 * executed. If this expression is not defined, no prompt is displayed to
	 * the user. Its default value is <code>null</code>.
	 * 
	 * @param prompt
	 *            Expression or <code>null</code> to disable the prompt.
	 */
	public void setPrompt(String prompt) {
		this.prompt = prompt;
	}

	/**
	 * Sets the task to execute.
	 * 
	 * @param task
	 *            Task to execute
	 */
	@Required
	public void setTask(GUITask<B, R> task) {
		this.task = task;
	}

	/**
	 * Sets the task executor used to execute the task
	 * 
	 * @param taskExecutor
	 *            Task executor
	 */
	@Required
	public void setTaskExecutor(GUITaskExecutor<B, R> taskExecutor) {
		this.taskExecutor = taskExecutor;
	}

	/**
	 * Sets if the input of the task must be the
	 * {@link ActionContext#getContext() contextual data} or the
	 * {@link ActionContext#getData() data} of the action context.
	 * 
	 * @param useContextData
	 *            <code>true</code> if the contextual data must be used.
	 * @deprecated Use
	 *             {@link AbstractGUIAction#setActionContextDataExtractor(net.sf.doolin.gui.action.support.ActionContextDataExtractor)}
	 */
	@Deprecated
	public void setUseContextData(boolean useContextData) {
		setActionContextDataExtractor(new DefaultActionContextDataExtractor(
				useContextData));
	}
}
