package net.sf.doolin.gui.action.path;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.action.path.item.ActionPath;
import net.sf.doolin.gui.action.path.list.ActionPathList;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Implementation for a list of {@link ActionContextPath}.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class DefaultActionContextPathList implements ActionContextPathList {

	private final static Logger log = LoggerFactory
			.getLogger(DefaultActionContextPathList.class);

	private final List<ActionContextPath> actionContextPathList;

	/**
	 * Constructor.
	 * 
	 * @param actionContextPathList
	 *            List of {@link ActionContextPath}.
	 */
	public DefaultActionContextPathList(
			List<ActionContextPath> actionContextPathList) {
		this.actionContextPathList = actionContextPathList;
	}

	/**
	 * Creates a menu bar context by merging several definitions
	 * 
	 * @param actionBarContextMap
	 *            Map of definitions
	 */
	public DefaultActionContextPathList(
			Map<String, ActionBarContext> actionBarContextMap) {
		// List of action paths to create
		this.actionContextPathList = new ArrayList<ActionContextPath>();
		// Indexes for replacement targets
		Map<String, AtomicInteger> placeholderTargetIndexes = new HashMap<String, AtomicInteger>();
		// For each definition
		for (ActionBarContext actionBarContext : actionBarContextMap.values()) {
			// Associated context
			ActionContext actionContext = actionBarContext.getActionContext();
			// List of action paths
			ActionPathList actionPaths = actionBarContext.getActionPaths();
			List<ActionPath> paths = actionPaths.getPaths();
			// For each action path
			for (ActionPath path : paths) {
				ActionContextPath actionContextPath = new ActionContextPath(
						actionContext, path);
				// Replacement target
				String target = path.getTarget();
				if (StringUtils.isNotBlank(target)) {
					this.actionContextPathList.add(actionContextPath);
					placeholderTargetIndexes.put(target, new AtomicInteger(
							this.actionContextPathList.size() - 1));
				} else {
					// Replacement source
					String source = path.getSource();
					if (StringUtils.isNotBlank(source)) {
						// Gets the target index
						AtomicInteger index = placeholderTargetIndexes
								.get(source);
						if (index == null) {
							log.error(String.format(
									"Cannot find any placeholder for %s",
									source));
						} else {
							this.actionContextPathList.add(index
									.getAndIncrement(), actionContextPath);
						}
					}
					// Standard
					else {
						this.actionContextPathList.add(actionContextPath);
					}
				}
			}
		}
		// Logger
		if (log.isDebugEnabled()) {
			log.debug(String.format("Action path context list:%n%s",
					this.actionContextPathList));
		}
	}

	@Override
	public List<ActionContextPath> getActionContextPathList() {
		return this.actionContextPathList;
	}

}
