package net.sf.doolin.gui.action.path.list;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.annotation.PostConstruct;

import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.action.path.item.ActionPath;
import net.sf.doolin.gui.action.path.item.GUIActionPath;
import net.sf.doolin.gui.action.path.item.PlaceholderActionPath;
import net.sf.doolin.gui.action.path.item.ReplacementActionPath;
import net.sf.doolin.gui.action.path.item.SeparatorActionPath;
import net.sf.doolin.gui.action.path.item.SubActionPath;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;

/**
 * This class defines a list of {@link ActionPath} from a list of encoded paths.
 * <p>
 * Each encoded path is a semi-colon-separated (;) list of tokens. Only the last
 * token is significant and will be used to create the actual {@link ActionPath}
 * type. The previous tokens are used to build the
 * {@linkplain ActionPath#getPathStack() path stack}.
 * </p>
 * <p>
 * For example, the "<code>File;New;-</code>" encoded path will create a
 * {@link SeparatorActionPath} (-) with the path stack set to ["File";"New"].
 * </p>
 * <p>
 * The table below shows the different formats used by the last token (which
 * identifies the actual {@link ActionPath} instance).
 * </p>
 * <table border="1">
 * <tr>
 * <th>Format</th>
 * <th>Type</th>
 * </tr>
 * <tr>
 * <td>-&lt;ignored&gt; (anything starting with a "-")</td>
 * <td>{@link SeparatorActionPath}. The remaining text is ignored.</td>
 * </tr>
 * <tr>
 * <td>@&lt;name&gt;</td>
 * <td>{@link PlaceholderActionPath} with the given
 * {@link PlaceholderActionPath#getName() name}</td>
 * </tr>
 * <tr>
 * <td>$&lt;name&gt;</td>
 * <td>{@link SubActionPath} whose {@link SubActionPath#getActionPath()
 * actionPath} is {@linkplain #getActionPath(String) looked up} in the
 * application context by its <code>name</code></td>
 * </tr>
 * <tr>
 * <td>&lt;name&gt;@&lt;placeholder&gt;</td>
 * <td>{@link ReplacementActionPath} with the given
 * {@link ReplacementActionPath#getPlaceholder() placeholder} and whose
 * {@link ReplacementActionPath#getAction() action} is
 * {@linkplain #getAction(String) looked up} in the application context by its
 * <code>name</code></td>
 * </tr>
 * <tr>
 * <td>&lt;name&gt;</td>
 * <td>Simple {@link GUIActionPath} whose {@link GUIActionPath#getAction()
 * action} is {@linkplain #getAction(String) looked up} in the application
 * context by its <code>name</code></td>
 * </tr>
 * </table>
 * 
 * @author Damien Coraboeuf
 * 
 */
public class ParsedActionPathList implements ActionPathList {

	@Autowired
	private ApplicationContext applicationContext;

	private List<String> paths = new ArrayList<String>(0);
	private List<ActionPath> actionPaths;

	/**
	 * Finds a {@link GUIAction} by its name in the {@link ApplicationContext}.
	 * 
	 * @param name
	 *            Action name
	 * @return {@link GUIAction} instance
	 */
	public GUIAction getAction(String name) {
		return (GUIAction) this.applicationContext.getBean(name);
	}

	/**
	 * Finds an {@link ActionPath} by its name in the {@link ApplicationContext}
	 * .
	 * 
	 * @param name
	 *            Action name
	 * @return {@link ActionPath} instance
	 */
	public ActionPath getActionPath(String name) {
		return (ActionPath) this.applicationContext.getBean(name);
	}

	@Override
	public List<ActionPath> getPaths() {
		return this.actionPaths;
	}

	/**
	 * Parses all encoded paths to actual instances of {@link ActionPath}.
	 */
	@PostConstruct
	public void init() {
		this.actionPaths = new ArrayList<ActionPath>();
		for (String path : this.paths) {
			if (StringUtils.isNotBlank(path)) {
				ActionPath actionPath = parse(path);
				this.actionPaths.add(actionPath);
			} else {
				this.actionPaths.add(new SeparatorActionPath());
			}
		}
	}

	/**
	 * Parses an encoded path to an actual instance of {@link ActionPath}.
	 * 
	 * @param path
	 *            Encoded path
	 * @return {@link ActionPath} instance
	 */
	protected ActionPath parse(String path) {
		// ";" separator
		String[] tokens = StringUtils.split(path, ";");
		int count = tokens.length;
		// Real path
		List<String> pathList = new ArrayList<String>();
		// For all tokens
		for (int i = 0; i < count; i++) {
			String token = tokens[i];
			if (i < count - 1) {
				pathList.add(token);
			} else {
				if (token.startsWith("-")) {
					return new SeparatorActionPath(pathList);
				} else if (token.startsWith("@")) {
					String name = StringUtils.substring(token, 1);
					return new PlaceholderActionPath(name, pathList);
				} else if (token.startsWith("$")) {
					String name = StringUtils.substring(token, 1);
					// Gets the action path
					ActionPath actionPath = getActionPath(name);
					// OK
					return new SubActionPath(actionPath, pathList);
				} else {
					if (token.indexOf('@') > 0) {
						String name = StringUtils.substringBefore(token, "@");
						GUIAction action = getAction(name);
						String placeholder = StringUtils.substringAfter(token,
								"@");
						return new ReplacementActionPath(action, placeholder,
								pathList);
					} else {
						GUIAction action = getAction(token);
						return new GUIActionPath(action, pathList);
					}
				}
			}
		}
		// No path indication
		List<String> emptyList = Collections.emptyList();
		return new SeparatorActionPath(emptyList);
	}

	/**
	 * Sets the list of paths using their encoded form.
	 * 
	 * @param paths
	 *            List of encoded paths
	 */
	public void setPaths(List<String> paths) {
		this.paths = paths;
	}

}
