package net.sf.doolin.gui.action.swing;

import java.util.List;
import java.util.Stack;

import javax.swing.Action;
import javax.swing.JMenuBar;
import javax.swing.JPopupMenu;
import javax.swing.JToolBar;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.action.path.ActionBarContext;
import net.sf.doolin.gui.action.path.ActionContextPath;
import net.sf.doolin.gui.action.path.ActionContextPathList;
import net.sf.doolin.gui.action.path.item.ActionPath;
import net.sf.doolin.gui.action.path.toolbar.ToolBar;
import net.sf.doolin.gui.action.support.SwingAction;
import net.sf.doolin.gui.service.IconSize;
import net.sf.doolin.gui.swing.JButtonBar;
import net.sf.doolin.gui.util.GUIStrings;

/**
 * Default action factory used in the Doolin GUI framework. It can be replaced
 * by declaring in the Spring configuration a bean that implements the
 * {@link ActionFactory} interface.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class DefaultActionFactory implements ActionFactory {

	/**
	 * Default instance
	 */
	public static final DefaultActionFactory INSTANCE = new DefaultActionFactory();

	@Override
	public Action createAction(ActionContext actionContext, GUIAction action,
			IconSize iconSize) {
		return new SwingAction(actionContext, action, iconSize);
	}

	@Override
	public JMenuBar createMenuBar(ActionContextPathList menuBarContext) {
		// Instance
		JMenuBar jMenuBar = new JMenuBar();
		JMenuBarBuilder menuBarBuilder = new JMenuBarBuilder(jMenuBar);
		// For each menu bar context path
		List<ActionContextPath> actionContextPathList = menuBarContext
				.getActionContextPathList();
		for (ActionContextPath actionContextPath : actionContextPathList) {
			ActionPath actionPath = actionContextPath.getActionPath();
			ActionContext actionContext = actionContextPath.getActionContext();
			installAction(menuBarBuilder, actionPath, actionContext);
		}
		// OK
		return jMenuBar;
	}

	/**
	 * Installs the actions defined by an {@link ActionBarContext} into the
	 * {@link MenuBuilder}.
	 * 
	 * @param menuBuilder
	 *            Menu builder to call
	 * @param actionBarContext
	 *            Definition of the action bar
	 */
	protected void installActionPaths(MenuBuilder menuBuilder,
			ActionBarContext actionBarContext) {
		// For each path definition
		for (ActionPath actionPath : actionBarContext.getActionPaths()
				.getPaths()) {
			// Installs the action
			installAction(menuBuilder, actionPath, actionBarContext
					.getActionContext());
		}
	}

	@Override
	public void installAction(MenuBuilder menuBuilder, ActionPath actionPath,
			ActionContext actionContext) {
		Stack<String> pathStack = actionPath.getPathStack();
		installAction(menuBuilder, pathStack, actionPath, actionContext);
	}

	/**
	 * Installs an action defined by its path into a {@link MenuBuilder}.
	 * 
	 * @param menuBuilder
	 *            Menu builder to call
	 * @param pathStack
	 *            Path to the action
	 * @param actionPath
	 *            Definition to the action
	 * @param actionContext
	 *            Action context
	 * @see ActionPath#install(MenuBuilder, ActionFactory, ActionContext)
	 */
	protected void installAction(MenuBuilder menuBuilder,
			Stack<String> pathStack, ActionPath actionPath,
			ActionContext actionContext) {
		// Installs the final component
		if (pathStack.isEmpty()) {
			actionPath.install(menuBuilder, this, actionContext);
		}
		// Installs a sub-menu
		else {
			String name = pathStack.pop();
			// Label
			String label = GUIStrings.get(name);
			// Creates the sub-menu
			MenuBuilder subMenuBuilder = menuBuilder.createSubMenu(name, label);
			// Goes on...
			installAction(subMenuBuilder, pathStack, actionPath, actionContext);
		}
	}

	@Override
	public JToolBar createToolBar(ActionBarContext actionBarContext) {
		// Tool bar
		JToolBar toolBar = new JToolBar();
		// Set-up
		Boolean floatable = (Boolean) actionBarContext
				.getProperty(ToolBar.FLOATABLE);
		if (floatable != null) {
			toolBar.setFloatable(floatable);
		}
		// Populates the button bar
		installActionPaths(new JToolBarBuilder(toolBar), actionBarContext);
		// OK
		return toolBar;
	}

	@Override
	public JButtonBar createButtonBar(ActionBarContext actionBarContext) {
		// Button bar
		JButtonBar buttonBar = new JButtonBar();
		// Populates the button bar
		installActionPaths(new JButtonBarBuilder(buttonBar), actionBarContext);
		// OK
		return buttonBar;
	}

	@Override
	public JPopupMenu createPopupMenu(ActionBarContext actionBarContext) {
		// Pop-up menu
		JPopupMenu popupMenu = new JPopupMenu();
		JPopupMenuBuilder menuBuilder = new JPopupMenuBuilder(popupMenu);
		// Populates the menu
		installActionPaths(menuBuilder, actionBarContext);
		// OK
		return popupMenu;
	}
}
