package net.sf.doolin.gui.action.task;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.monitor.Task;
import net.sf.doolin.gui.monitor.TaskException;
import net.sf.doolin.gui.monitor.TaskMonitor;

import org.springframework.beans.factory.annotation.Required;

/**
 * Runs a {@link GUITask} whose execution is monitored by a
 * {@linkplain TaskMonitor modal dialog}.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of bean sent to the task
 * @param <R>
 *            Type of bean returned by the task
 * @see TaskMonitor
 */
public class MonitoredGUITaskExecutor<B, R> implements GUITaskExecutor<B, R> {

	private String titleExpression;

	private String messageExpression;

	public void execute(final ActionContext actionContext, final B bean,
			final GUITask<B, R> guiTask) throws TaskException {
		// Task
		Task<R, Void> task = new Task<R, Void>() {

			@Override
			protected R doInBackground() throws Exception {
				return guiTask.doInBackground(actionContext, bean);
			}
		};
		// Monitor title
		String title = new GUIExpression(
				actionContext.getSubscriberValidator(), bean,
				this.titleExpression).getValue();
		String message = new GUIExpression(actionContext
				.getSubscriberValidator(), bean, this.messageExpression)
				.getValue();
		// Monitor
		TaskMonitor<R, Void> monitor = new TaskMonitor<R, Void>(task,
				actionContext.getComponent(), title, message, 0);
		// Execution
		try {
			// Starts the monitor
			R result = monitor.start();
			// Uses the result
			guiTask.doOnResult(actionContext, bean, result);
		} catch (TaskException ex) {
			// Uses the exception
			guiTask.doOnException(actionContext, bean, ex.getCause());
		}
	}

	/**
	 * Sets the expression for the message to display in the monitor. Its model
	 * is the input bean of the task.
	 * 
	 * @param messageExpression
	 *            Expression
	 */
	@Required
	public void setMessageExpression(String messageExpression) {
		this.messageExpression = messageExpression;
	}

	/**
	 * Sets the expression for the title to display in the monitor. Its model is
	 * the input bean of the task.
	 * 
	 * @param titleExpression
	 *            Expression
	 */
	@Required
	public void setTitleExpression(String titleExpression) {
		this.titleExpression = titleExpression;
	}

}
