package net.sf.doolin.gui.display;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.bus.support.SubscriberValidator;
import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.util.Utils;

import org.springframework.beans.factory.annotation.Required;

/**
 * Handler that evaluates a condition on a property of the context.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class ConditionalDisplayStateHandler extends AbstractDisplayStateHandler {

	private String propertyPath;
	private ConditionalOperator operator = ConditionalOperator.NOT_NULL;
	private Object operand;
	private String operandPath;
	private String operandValue;
	private DisplayState ifTrueState = DisplayState.ENABLED;
	private DisplayState ifFalseState = DisplayState.DISABLED;
	private boolean readOnly = false;

	@Override
	public DisplayState getDisplayState(ActionContext actionContext) {
		Object data = actionContext.getData();
		Object value = Utils.getProperty(data, this.propertyPath);
		Object otherValue = getOtherValue(data);
		boolean result = this.operator.evaluate(value, otherValue);
		return result ? this.ifTrueState : this.ifFalseState;
	}

	/**
	 * Gets the if false state.
	 * 
	 * @return the if false state
	 */
	public DisplayState getIfFalseState() {
		return this.ifFalseState;
	}

	/**
	 * Gets the if true state.
	 * 
	 * @return the if true state
	 */
	public DisplayState getIfTrueState() {
		return this.ifTrueState;
	}

	/**
	 * Gets the operand.
	 * 
	 * @return the operand
	 */
	public Object getOperand() {
		return this.operand;
	}

	/**
	 * Gets the path to the property that contains the operand.
	 * 
	 * @return Property path
	 */
	public String getOperandPath() {
		return this.operandPath;
	}

	/**
	 * Gets the operand value as a {@link String}.
	 * 
	 * @return Operand value
	 */
	public String getOperandValue() {
		return this.operandValue;
	}

	/**
	 * Gets the operator.
	 * 
	 * @return the operator
	 */
	public ConditionalOperator getOperator() {
		return this.operator;
	}

	/**
	 * Gets the property path.
	 * 
	 * @return the property path
	 */
	public String getPropertyPath() {
		return this.propertyPath;
	}

	/**
	 * Returns if the target property is read-only
	 * 
	 * @return <code>true</code> if the target property is read-only
	 */
	public boolean isReadOnly() {
		return this.readOnly;
	}

	/**
	 * Display state to return when condition is evaluated to <code>false</code>
	 * 
	 * @param ifFalseState
	 *            Display state to return when condition is evaluated to true
	 */
	public void setIfFalseState(DisplayState ifFalseState) {
		this.ifFalseState = ifFalseState;
	}

	/**
	 * Display state to return when condition is evaluated to <code>true</code>
	 * 
	 * @param ifTrueState
	 *            Display state to return when condition is evaluated to true
	 */
	public void setIfTrueState(DisplayState ifTrueState) {
		this.ifTrueState = ifTrueState;
	}

	/**
	 * Sets the operand for the operator.
	 * 
	 * @param operand
	 *            the new operand
	 */
	public void setOperand(Object operand) {
		this.operand = operand;
	}

	/**
	 * Sets the path to the property that contains the operand value
	 * 
	 * @param operandPath
	 *            Property path
	 */
	public void setOperandPath(String operandPath) {
		this.operandPath = operandPath;
	}

	/**
	 * Sets the operand value as a {@link String}.
	 * 
	 * @param operandValue
	 *            Operand value
	 */
	public void setOperandValue(String operandValue) {
		this.operandValue = operandValue;
	}

	/**
	 * Sets the operator to use.
	 * 
	 * @param operator
	 *            the new operator
	 */
	public void setOperator(ConditionalOperator operator) {
		this.operator = operator;
	}

	/**
	 * Sets the path to the property in the context.
	 * 
	 * @param propertyPath
	 *            the new property path
	 */
	@Required
	public void setPropertyPath(String propertyPath) {
		this.propertyPath = propertyPath;
	}

	/**
	 * Sets if the target property is read-only
	 * 
	 * @param readOnly
	 *            <code>true</code> if the target property is read-only
	 */
	public void setReadOnly(boolean readOnly) {
		this.readOnly = readOnly;
	}

	@Override
	public void subscribe(ActionContext actionContext, Runnable task) {
		SubscriberValidator subscriberValidator = actionContext
				.getSubscriberValidator();
		PropertyChangeSupport.subscribe(subscriberValidator, actionContext
				.getData(), this.propertyPath, task);
		if (this.operandPath != null) {
			PropertyChangeSupport.subscribe(subscriberValidator, actionContext
					.getData(), this.operandPath, task);
		}
	}

	/**
	 * Gets the value for the operand. The order of priority is the following:
	 * <ol>
	 * <li>Gets the {@linkplain #getOperandValue() operand value} specified as a
	 * {@link String}
	 * <li>Gets the operand value defined by a {@linkplain #getOperandPath()
	 * property} on the given data
	 * <li>Gets the {@linkplain #getOperand() direct operand value}
	 * </ol>
	 * 
	 * @param data
	 *            Bean to get the property from (if needed)
	 * @return Operand value to actually use
	 */
	protected Object getOtherValue(Object data) {
		if (this.operandValue != null) {
			return this.operandValue;
		} else if (this.operandPath != null) {
			return Utils.getProperty(data, this.operandPath);
		} else {
			return this.operand;
		}
	}

}
