package net.sf.doolin.gui.display;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.bus.support.SubscriberValidator;
import net.sf.doolin.gui.action.ActionContext;
import net.sf.jstring.LocalizableException;
import org.apache.commons.jexl.Expression;
import org.apache.commons.jexl.ExpressionFactory;
import org.apache.commons.jexl.JexlContext;
import org.apache.commons.jexl.JexlHelper;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Required;

/**
 * Handler that evaluates a condition on a property of the context.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class ConditionalJEXLDisplayStateHandler extends
		AbstractDisplayStateHandler {

	private String propertyPath;
	private String condition;
	private DisplayState ifTrueState = DisplayState.ENABLED;
	private DisplayState ifFalseState = DisplayState.DISABLED;

	public String getCondition() {
		return this.condition;
	}

	@SuppressWarnings("unchecked")
	@Override
	public DisplayState getDisplayState(ActionContext actionContext) {
		Object data = actionContext.getData();
		try {
			Expression e = ExpressionFactory.createExpression(this.condition);
			JexlContext context = JexlHelper.createContext();
			context.getVars().put("model", data);
			Object o = e.evaluate(context);
			if (o == null) {
				return this.ifFalseState;
			} else if (o instanceof Boolean) {
				return getState((Boolean) o);
			} else {
				return getState(Boolean.valueOf(o.toString()));
			}
		} catch (Exception ex) {
			throw new LocalizableException("ConditionalJEXLDisplayStateHandler.error",
					ex, this.condition, ex);
		}
	}

	/**
	 * Gets the if false state.
	 * 
	 * @return the if false state
	 */
	public DisplayState getIfFalseState() {
		return this.ifFalseState;
	}

	/**
	 * Gets the if true state.
	 * 
	 * @return the if true state
	 */
	public DisplayState getIfTrueState() {
		return this.ifTrueState;
	}

	/**
	 * Gets the property path.
	 * 
	 * @return the property path
	 */
	public String getPropertyPath() {
		return this.propertyPath;
	}

	private DisplayState getState(boolean o) {
		return o ? this.ifTrueState : this.ifFalseState;
	}

	/**
	 * Sets the JEXL condition, where 'model' is set for the context data
	 * 
	 * @param condition
	 *            JEXL expression
	 */
	@Required
	public void setCondition(String condition) {
		this.condition = condition;
	}

	/**
	 * Display state to return when condition is evaluated to <code>false</code>
	 * 
	 * @param ifFalseState
	 *            Display state to return when condition is evaluated to true
	 */
	public void setIfFalseState(DisplayState ifFalseState) {
		this.ifFalseState = ifFalseState;
	}

	/**
	 * Display state to return when condition is evaluated to <code>true</code>
	 * 
	 * @param ifTrueState
	 *            Display state to return when condition is evaluated to true
	 */
	public void setIfTrueState(DisplayState ifTrueState) {
		this.ifTrueState = ifTrueState;
	}

	/**
	 * Sets the path to the property in the context.
	 * 
	 * @param propertyPath
	 *            the new property path
	 */
	@Required
	public void setPropertyPath(String propertyPath) {
		this.propertyPath = propertyPath;
	}

	@Override
	public void subscribe(ActionContext actionContext, Runnable task) {
		if (StringUtils.isNotBlank(this.propertyPath)) {
			SubscriberValidator subscriberValidator = actionContext
					.getSubscriberValidator();
			PropertyChangeSupport.subscribe(subscriberValidator, actionContext
					.getData(), this.propertyPath, task);
		}
	}

}
