package net.sf.doolin.gui.field.table;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;

import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import net.sf.doolin.bus.bean.Bean;
import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.field.table.column.ColumnEditable;
import net.sf.doolin.gui.field.table.column.ColumnUpdatePolicy;
import net.sf.doolin.util.Utils;
import net.sf.doolin.util.unit.ValueUnit;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Required;

/**
 * Column definition.
 * 
 * @author Damien Coraboeuf
 */
public class Column extends Bean {

	private static final long serialVersionUID = 1L;

	/**
	 * Filter.
	 * 
	 * @param allColumns
	 *            the all columns
	 * 
	 * @return the list< column>
	 */
	public static List<Column> filter(List<Column> allColumns) {
		ArrayList<Column> filteredList = new ArrayList<Column>();
		for (Column column : allColumns) {
			if (column.isVisible()) {
				filteredList.add(column);
			}
		}
		return filteredList;
	}

	private String name;

	private String titleExpression;

	private ColumnValueFactory valueFactory;

	private ValueUnit width;

	private boolean resizeable = true;

	private String type;

	private TableCellRenderer renderer;

	private ColumnEditable<Object> editable;

	private TableCellEditor editor;

	private boolean visible = true;

	private boolean required = false;

	private GUIAction onEditAction;

	private ColumnUpdatePolicy columnUpdatePolicy = ColumnUpdatePolicy.CELL;

	/**
	 * Predefined classes
	 */
	private static HashMap<String, Class<?>> classes = new HashMap<String, Class<?>>();

	static {
		classes.put("boolean", Boolean.class);
		classes.put("int", Integer.class);
		classes.put("string", String.class);
	}
	private Class<?> columnClass;

	/**
	 * Creates a copy of this column.
	 * 
	 * @return Copy
	 */
	public Column copy() {
		Column c = new Column();
		Utils.copyProperties(c, this);
		return c;
	}

	/**
	 * Class for the column.
	 * 
	 * @return the column class
	 */
	public Class<?> getColumnClass() {
		if (this.columnClass == null) {
			if (StringUtils.isBlank(this.type)) {
				return Object.class;
			} else {
				this.columnClass = classes.get(this.type);
				if (this.columnClass == null) {
					this.columnClass = Utils.forClass(this.type);
				}
			}
		}
		return this.columnClass;
	}

	/**
	 * Gets the column update policy.
	 * 
	 * @return the column update policy
	 */
	public ColumnUpdatePolicy getColumnUpdatePolicy() {
		return this.columnUpdatePolicy;
	}

	/**
	 * Returns the comparator which is used by this column.
	 * 
	 * @return Comparator
	 * @see ColumnValueFactory#getComparator()
	 */
	public Comparator<?> getComparator() {
		return this.valueFactory.getComparator();
	}

	/**
	 * Gets the editable.
	 * 
	 * @return the editable
	 */
	public ColumnEditable<Object> getEditable() {
		return this.editable;
	}

	/**
	 * Gets the editor.
	 * 
	 * @return the editor
	 */
	public TableCellEditor getEditor() {
		return this.editor;
	}

	/**
	 * Gets the name.
	 * 
	 * @return the name
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Gets the on edit action.
	 * 
	 * @return the on edit action
	 */
	public GUIAction getOnEditAction() {
		return this.onEditAction;
	}

	/**
	 * Gets the renderer.
	 * 
	 * @return the renderer
	 */
	public TableCellRenderer getRenderer() {
		return this.renderer;
	}

	/**
	 * Gets the title expression.
	 * 
	 * @return the title expression
	 */
	public String getTitleExpression() {
		return this.titleExpression;
	}

	/**
	 * Gets the type.
	 * 
	 * @return the type
	 */
	public String getType() {
		return this.type;
	}

	/**
	 * Gets the value factory.
	 * 
	 * @return the value factory
	 */
	public ColumnValueFactory getValueFactory() {
		return this.valueFactory;
	}

	/**
	 * Gets the width.
	 * 
	 * @return the width
	 */
	public ValueUnit getWidth() {
		return this.width;
	}

	/**
	 * Returns if the object is editable or not for this column.
	 * 
	 * @param baseObject
	 *            Object to edit
	 * 
	 * @return <code>true</code> if this object is editable or not
	 * 
	 * @see ColumnEditable#isEditable(Column, Object)
	 */
	public boolean isEditable(Object baseObject) {
		if (this.editable != null) {
			return this.editable.isEditable(this, baseObject);
		} else {
			return false;
		}
	}

	/**
	 * Checks if the display of this column is required or not.
	 * 
	 * @return <code>true</code> if this column is required to be displayed
	 */
	public boolean isRequired() {
		return this.required;
	}

	/**
	 * Checks if is resizeable.
	 * 
	 * @return true, if is resizeable
	 */
	public boolean isResizeable() {
		return this.resizeable;
	}

	/**
	 * Checks if is visible.
	 * 
	 * @return true, if is visible
	 */
	public boolean isVisible() {
		return this.visible;
	}

	/**
	 * Update policy when the column cell is edited
	 * 
	 * @param columnUpdatePolicy
	 *            the new column update policy
	 */
	public void setColumnUpdatePolicy(ColumnUpdatePolicy columnUpdatePolicy) {
		notify("columnUpdatePolicy", this.columnUpdatePolicy,
				this.columnUpdatePolicy = columnUpdatePolicy);
	}

	/**
	 *Defines if the column is editable
	 * 
	 * @param editable
	 *            the new editable
	 */
	public void setEditable(ColumnEditable<Object> editable) {
		notify("editable", this.editable, this.editable = editable);
	}

	/**
	 * Editor for the column
	 * 
	 * @param editor
	 *            the new editor
	 */
	public void setEditor(TableCellEditor editor) {
		notify("editor", this.editor, this.editor = editor);
	}

	/**
	 * Name of the column
	 * 
	 * @param name
	 *            the new name
	 */
	@Required
	public void setName(String name) {
		notify("name", this.name, this.name = name);
	}

	/**
	 * Action to execute when an item is edited
	 * 
	 * @param onEditAction
	 *            the on edit action
	 */
	public void setOnEditAction(GUIAction onEditAction) {
		notify("onEditAction", this.onEditAction,
				this.onEditAction = onEditAction);
	}

	/**
	 *Renderer for the column
	 * 
	 * @param renderer
	 *            the new renderer
	 */
	public void setRenderer(TableCellRenderer renderer) {
		notify("renderer", this.renderer, this.renderer = renderer);
	}

	/**
	 * Sets if the display of this column is required or not. It is set to
	 * <code>true</code> by default.
	 * 
	 * @param required
	 *            <code>true</code> if this column is required to be displayed
	 */
	public void setRequired(boolean required) {
		this.required = required;
	}

	/**
	 * Defines if the column can be resized
	 * 
	 * @param resizeable
	 *            the new resizeable
	 */
	public void setResizeable(boolean resizeable) {
		notify("resizeable", this.resizeable, this.resizeable = resizeable);
	}

	/**
	 * Sets the title expression.
	 * 
	 * @param titleExpression
	 *            the new title expression
	 */
	@Required
	public void setTitleExpression(String titleExpression) {
		notify("titleExpression", this.titleExpression,
				this.titleExpression = titleExpression);
	}

	/**
	 * Sets the type for the column.
	 * 
	 * @param type
	 *            the new type
	 */
	public void setType(String type) {
		notify("type", this.type, this.type = type);
	}

	/**
	 * Sets the factory for the value of the column.
	 * 
	 * @param valueFactory
	 *            the new value factory
	 */
	@Required
	public void setValueFactory(ColumnValueFactory valueFactory) {
		notify("valueFactory", this.valueFactory,
				this.valueFactory = valueFactory);
	}

	/**
	 * Sets the initial visibility of the column.
	 * 
	 * @param visible
	 *            the new visible
	 */
	public void setVisible(boolean visible) {
		notify("visible", this.visible, this.visible = visible);
	}

	/**
	 * Sets the default width of the column.
	 * 
	 * @param width
	 *            the new width
	 */
	public void setWidth(ValueUnit width) {
		notify("width", this.width, this.width = width);
	}

	@Override
	public String toString() {
		return this.name;
	}

}
