package net.sf.doolin.gui.field.table.column;

import java.util.List;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.bus.support.SubscriberValidator;
import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.field.table.Column;
import net.sf.doolin.gui.field.table.ColumnGenerator;
import net.sf.doolin.gui.field.table.support.GUITableField;
import net.sf.doolin.gui.service.GUIPreferences;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Required;

import com.google.common.base.Function;
import com.google.common.collect.Lists;

/**
 * Column generated that holds a fixed list of columns.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 * @param <E>
 *            Type of object for the item list
 */
public class DefaultColumnGenerator<V, E> implements ColumnGenerator<V, E> {

	/**
	 * Property name for the columns: {@value}
	 */
	public static final String FIELD_COLUMNS = "columns";

	private List<Column> columns;

	@Override
	public List<Column> generateColumns(GUITableField<V, E> tableField) {
		// Gets the columns of the field
		@SuppressWarnings("unchecked")
		List<Column> list = (List<Column>) tableField
				.getProperty(FIELD_COLUMNS);
		if (list != null) {
			return list;
		}
		// Creates the initial list
		list = Lists.transform(this.columns, new Function<Column, Column>() {

			@Override
			public Column apply(Column source) {
				return source.copy();
			}
		});
		// Filter using preferences?
		String name = tableField.getTableDescriptor().getName();
		if (StringUtils.isNotBlank(name)) {
			ActionContext actionContext = tableField.getActionContext();
			SubscriberValidator validator = actionContext
					.getSubscriberValidator();
			// Gets the preferences
			final GUIPreferences preferences = actionContext.getApplication()
					.getPreferences();
			// For all columns
			for (Column column : list) {
				String columnName = column.getName();
				if (StringUtils.isBlank(columnName)) {
					continue;
				}
				// Visibility
				final String visibleKey = String.format("%s.%s.visible", name,
						columnName);
				boolean visible = preferences.getBoolean(visibleKey, column
						.isVisible());
				column.setVisible(visible);
				// Registers to column changes
				final Column thisColumn = column;
				PropertyChangeSupport.subscribe(validator, thisColumn, null,
						new Runnable() {

							@Override
							public void run() {
								preferences.setBoolean(visibleKey, thisColumn
										.isVisible());
								preferences.save();
							}
						});
			}
		}
		// Adds to the field
		tableField.setProperty(FIELD_COLUMNS, list);
		// OK
		return list;
	}

	/**
	 * Gets the columns.
	 * 
	 * @return the columns
	 */
	public List<Column> getColumns() {
		return this.columns;
	}

	/**
	 * Sets the columns.
	 * 
	 * @param columns
	 *            the new columns
	 */
	@Required
	public void setColumns(List<Column> columns) {
		this.columns = columns;
	}

}
