package net.sf.doolin.gui.form;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.TransferHandler;

import net.sf.doolin.gui.display.DisplayStateHandler;
import net.sf.doolin.gui.display.DisplayStateHandlerUtils;
import net.sf.doolin.gui.field.Field;
import net.sf.doolin.gui.field.FieldDescriptor;
import net.sf.doolin.gui.field.support.FieldTransferHandler;
import net.sf.doolin.gui.layout.LayoutService;
import net.sf.doolin.gui.style.Style;
import net.sf.doolin.gui.validation.GUIValidationDecorator;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.support.GUIViewAdapter;

/**
 * Descriptor for a panel that contains a list of fields.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 */
public class FormPanel<V> extends JPanel {

	private static final long serialVersionUID = 1L;

	/**
	 * Field property that contains the decorated component of the field
	 */
	protected static final String FIELD_COMPONENT_PROPERTY = "fieldComponent";

	private final LayoutService layout;
	private final GUIValidationDecorator validationDecorator;
	private final List<FieldDescriptor<V>> fieldDescriptorList;

	private final List<Field<V>> fieldList;

	/**
	 * Constructor
	 * 
	 * @param layout
	 *            Layout service to use
	 * @param validationDecorator
	 *            Decorator used for the validation
	 * @param fieldDescriptorList
	 *            List of fields
	 */
	public FormPanel(LayoutService layout,
			GUIValidationDecorator validationDecorator,
			List<FieldDescriptor<V>> fieldDescriptorList) {
		this.layout = layout;
		this.validationDecorator = validationDecorator;
		this.fieldDescriptorList = fieldDescriptorList;
		this.fieldList = new ArrayList<Field<V>>();
	}

	/**
	 * Adds the field.
	 * 
	 * @param view
	 *            the view
	 * @param fieldDescriptor
	 *            the field descriptor
	 */
	protected void addField(GUIView<V> view,
			final FieldDescriptor<V> fieldDescriptor) {
		// Creates the field
		final Field<V> field = fieldDescriptor.createField(view);
		// Applies the style if any
		Style style = fieldDescriptor.getStyle();
		if (style != null) {
			style.apply(field.getFieldComponent());
		}
		// Layout constraint
		String constraint = fieldDescriptor.getConstraint();
		// Field component
		JComponent fieldComponent = field.getFieldComponent();
		JComponent coreFieldComponent = field.getCoreComponent();
		// Transfer handler
		TransferHandler transferHandler = fieldDescriptor.getTransferHandler();
		if (transferHandler != null) {
			if (transferHandler instanceof FieldTransferHandler<?>) {
				@SuppressWarnings("unchecked")
				FieldTransferHandler<V> fieldTransferHandler = (FieldTransferHandler<V>) transferHandler;
				fieldTransferHandler.setField(field);
			}
			coreFieldComponent.setTransferHandler(transferHandler);
		}
		// Decorates the field component
		if (this.validationDecorator != null) {
			fieldComponent = this.validationDecorator.decorateField(field,
					fieldComponent);
		}
		// Set-up of properties
		field.setProperty(FIELD_COMPONENT_PROPERTY, fieldComponent);
		this.fieldList.add(field);
		// Positions on the panel
		this.layout.add(this, fieldComponent, constraint);
	}

	/**
	 * Initialises the content of the panel
	 * 
	 * @param view
	 *            Hosting view
	 */
	public void init(GUIView<V> view) {
		// Configures the layout
		this.layout.init(this);
		// For each field
		for (FieldDescriptor<V> field : this.fieldDescriptorList) {
			// Configure the field
			addField(view, field);
		}
		// Listens to events on this view
		view.addViewListener(new GUIViewAdapter<V>() {

			@Override
			public void onViewInit(GUIView<V> view) {
				// For each field
				for (Field<V> field : FormPanel.this.fieldList) {
					// Shows the field
					field.getFieldDescriptor().onInit(field);
				}
			}

			@Override
			public void onViewShown(GUIView<V> view) {
				// For each field
				for (Field<V> field : FormPanel.this.fieldList) {
					// Shows the field
					field.getFieldDescriptor().onShow(field);
				}
			}

		});
	}

	/**
	 * Initialises the state of all fields
	 * 
	 * @param view
	 *            Hosting view
	 */
	public void initState(GUIView<V> view) {
		// For each field
		for (Field<V> field : this.fieldList) {
			// Field decorated component
			JComponent fieldComponent = (JComponent) field
					.getProperty(FIELD_COMPONENT_PROPERTY);
			// Display state handler
			DisplayStateHandler displayStateHandler = field
					.getFieldDescriptor().getDisplayStateHandler();
			// Set-up
			if (displayStateHandler != null) {
				setupDisplayState(field, fieldComponent, displayStateHandler);
			}
		}
	}

	/**
	 * Setup the display state.
	 * 
	 * @param field
	 *            the field
	 * @param component
	 *            Decorated field component
	 * @param displayStateHandler
	 *            the display state handler
	 * @see DisplayStateHandlerUtils#setupField(DisplayStateHandler, Field,
	 *      JComponent)
	 */
	protected void setupDisplayState(Field<V> field, JComponent component,
			DisplayStateHandler displayStateHandler) {
		DisplayStateHandlerUtils.setupField(displayStateHandler, field,
				component);
	}

}
