package net.sf.doolin.gui.form;

import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.field.AbstractFieldDescriptor;
import net.sf.doolin.gui.field.Field;
import net.sf.doolin.gui.field.FieldDescriptor;
import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.layout.FormLayoutService;
import net.sf.doolin.gui.layout.LayoutService;
import net.sf.doolin.gui.swing.CollapsablePanel;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.descriptor.GUIViewDescriptor;
import net.sf.doolin.util.Utils;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Required;

/**
 * Field that contains other fields.
 * 
 * @param <V>
 *            Type of the data for the view
 * 
 * @author Damien Coraboeuf
 */
public class FormSection<V> extends AbstractFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private static final String FORM_PANEL_PROPERTY = "FormPanel";
	private boolean border = true;
	private boolean collapsable = false;
	private boolean collapsed = true;
	private String collapsedPropertyPath;

	private String titleExpression;
	private LayoutService layout = new FormLayoutService("default:grow",
			"default");
	private List<FieldDescriptor<V>> fieldList = new ArrayList<FieldDescriptor<V>>(
			0);

	@Override
	public Field<V> createField(final GUIView<V> view) {
		// Gets the form descriptor
		GUIViewDescriptor<V> viewDescriptor = view.getViewDescriptor();
		if (!(viewDescriptor instanceof FormViewDescriptor<?>)) {
			throw new IllegalStateException(
					"Form sections cannot be used outside of a form view");
		}
		FormViewDescriptor<V> formDescriptor = (FormViewDescriptor<V>) viewDescriptor;
		// Panel
		FormPanel<V> formPanel = new FormPanel<V>(this.layout, formDescriptor
				.getValidationDecorator(), this.fieldList);
		// Initialises the panel
		formPanel.init(view);
		// Border
		Border border = null;
		if (this.border) {
			// Line border
			border = BorderFactory.createEtchedBorder();
		}
		// Title border
		if (StringUtils.isNotBlank(this.titleExpression)) {
			GUIExpression expression = new GUIExpression(view
					.getActionContext().getSubscriberValidator(), view
					.getViewData(), this.titleExpression);
			TitledBorder titledBorder = BorderFactory.createTitledBorder(
					border, expression.getValue());
			expression.connect(
					view.getActionContext().getSubscriberValidator(),
					titledBorder, "title");
			border = titledBorder;
		}
		// Collapsable
		if (this.collapsable) {
			final CollapsablePanel collapsablePanel = new CollapsablePanel(
					formPanel, border);
			// Initial state
			collapsablePanel.setCollapsed(this.collapsed);
			// Connection
			if (StringUtils.isNotBlank(this.collapsedPropertyPath)) {
				Runnable collapsedTask = new Runnable() {

					@Override
					public void run() {
						boolean collapsed = (Boolean) Utils.getProperty(view
								.getViewData(),
								FormSection.this.collapsedPropertyPath);
						collapsablePanel.setCollapsed(collapsed);
					}
				};
				PropertyChangeSupport.subscribe(view.getActionContext()
						.getSubscriberValidator(), view.getViewData(),
						this.collapsedPropertyPath, collapsedTask);
			}
			// FormPanel as property
			SimpleField<V> field = new SimpleField<V>(view, this,
					collapsablePanel);
			field.setProperty(FORM_PANEL_PROPERTY, formPanel);
			// OK
			return field;
		} else {
			// Border OK
			if (border != null) {
				formPanel.setBorder(border);
			}
			// FormPanel as property
			SimpleField<V> field = new SimpleField<V>(view, this, formPanel);
			field.setProperty(FORM_PANEL_PROPERTY, formPanel);
			// OK
			return field;
		}
	}

	/**
	 * Gets the path to the property that defines the collapsed state
	 * 
	 * @return Property path or <code>null</code> if not defined
	 */
	public String getCollapsedPropertyPath() {
		return this.collapsedPropertyPath;
	}

	/**
	 * Gets the field list.
	 * 
	 * @return the field list
	 */
	public List<FieldDescriptor<V>> getFieldList() {
		return this.fieldList;
	}

	/**
	 * Gets the layout.
	 * 
	 * @return the layout
	 */
	public LayoutService getLayout() {
		return this.layout;
	}

	/**
	 * Gets the title expression.
	 * 
	 * @return the title expression
	 */
	public String getTitleExpression() {
		return this.titleExpression;
	}

	/**
	 * Checks if is border.
	 * 
	 * @return true, if is border
	 */
	public boolean isBorder() {
		return this.border;
	}

	/**
	 * Checks if the section can be collapsed or not
	 * 
	 * @return <code>true</code> if the section can be collapsed
	 */
	public boolean isCollapsable() {
		return this.collapsable;
	}

	/**
	 * Checks if the section is collapsed
	 * 
	 * @return <code>true</code> if the section is collapsed
	 */
	public boolean isCollapsed() {
		return this.collapsed;
	}

	@Override
	public void onInit(Field<V> field) {
		@SuppressWarnings("unchecked")
		FormPanel<V> formPanel = (FormPanel<V>) field
				.getProperty(FORM_PANEL_PROPERTY);
		formPanel.initState(field.getView());
		super.onInit(field);
	}

	/**
	 * Sets <code>true</code> if a border must surround the section.
	 * 
	 * @param border
	 *            the new border
	 */
	public void setBorder(boolean border) {
		this.border = border;
	}

	/**
	 * Sets if the section can be collapsed or not
	 * 
	 * @param collapsable
	 *            <code>true</code> if the section can be collapsed
	 */
	public void setCollapsable(boolean collapsable) {
		this.collapsable = collapsable;
	}

	/**
	 * Sets if the section is collapsed
	 * 
	 * @param collapsed
	 *            <code>true</code> if the section is collapsed
	 */
	public void setCollapsed(boolean collapsed) {
		this.collapsed = collapsed;
	}

	/**
	 * Sets the path to the property that defines the collapsed state
	 * 
	 * @param collapsedPropertyPath
	 *            Property path or <code>null</code> if not defined
	 */
	public void setCollapsedPropertyPath(String collapsedPropertyPath) {
		this.collapsedPropertyPath = collapsedPropertyPath;
	}

	/**
	 * Sets the field list.
	 * 
	 * @param fieldList
	 *            the new field list
	 */
	@Required
	public void setFieldList(List<FieldDescriptor<V>> fieldList) {
		this.fieldList = fieldList;
	}

	/**
	 * Sets the layout.
	 * 
	 * @param layout
	 *            the new layout
	 */
	@Required
	public void setLayout(LayoutService layout) {
		this.layout = layout;
	}

	/**
	 * Sets the title expression for the border title.
	 * 
	 * @param titleExpression
	 *            the new title expression
	 */
	public void setTitleExpression(String titleExpression) {
		this.titleExpression = titleExpression;
	}

}
