/*
 * Created on 16 oct. 2005
 */
package net.sf.doolin.gui.service.icon;

import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.util.Map;

import net.sf.doolin.gui.service.IconService;
import net.sf.doolin.gui.service.IconSize;
import net.sf.doolin.util.Utils;

import org.apache.commons.lang.StringUtils;

/**
 * URL stream handler that handles the "icon:" protocol by calling a
 * {@link IconService} instance.
 * <p>
 * The URL has the form:
 * 
 * <pre>
 * icon:key?param=value&amp;param=value&amp;...
 * </pre>
 * 
 * where <code>key</code> is used as the icon key and parameters can be:
 * <ul>
 * <li><code>size</code> - {@link IconSize} for the icon (
 * {@link IconSize#MEDIUM} by default)
 * </ul>
 * 
 * @author Damien Coraboeuf
 */
public class IconURLStreamHandler extends URLStreamHandler {

	/**
	 * Action protocol
	 */
	public static final String PROTOCOL_ICON = "icon";

	/**
	 * Icon manager
	 */
	private final IconService iconService;

	/**
	 * Constructor
	 * 
	 * @param iconService
	 *            Icon service to call.
	 */
	public IconURLStreamHandler(IconService iconService) {
		this.iconService = iconService;
	}

	/**
	 * @see java.net.URLStreamHandler#openConnection(java.net.URL)
	 */
	@Override
	protected URLConnection openConnection(URL u) throws IOException {
		// ID of the Icon to load
		String iconId = u.getPath();
		// Parameters
		Map<String, String> request = Utils.parseQuery(u);
		// Size of the icon
		IconSize size = IconSize.MEDIUM;
		String paramSize = request.get("size");
		if (StringUtils.isNotBlank(paramSize)) {
			paramSize = StringUtils.upperCase(paramSize);
			size = IconSize.valueOf(paramSize);
		}
		// Loads the icon
		URL iconURL = iconService.getIconURL(iconId, size);
		if (iconURL == null) {
			return null;
		} else {
			return iconURL.openConnection();
		}
	}

}
