/*
 * Created on Jul 17, 2007
 */
package net.sf.doolin.gui.service.support;

import java.io.File;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import net.sf.doolin.gui.app.GUIApplication;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Implementation of preferences based on default JDK preferences
 * implementation.
 * 
 * @see net.sf.doolin.gui.service.GUIPreferences
 * @author Damien Coraboeuf
 */
public class DefaultGUIPreferences extends AbstractGUIPreferences {

	/**
	 * Logger
	 */
	private final static Logger log = LoggerFactory
			.getLogger(DefaultGUIPreferences.class);

	private Preferences preferences;

	/**
	 * Initializes the preferences.
	 * 
	 */
	protected void init() {
		if (this.preferences == null) {
			syncInit();
		}
	}

	/**
	 * Actual initialization. This method is called only once in the life-cycle
	 * of the preferences and is synchronized.
	 * 
	 */
	protected synchronized void syncInit() {
		if (this.preferences == null) {
			Class<?> idClass = GUIApplication.getIdClass();
			Preferences prefs = Preferences.userNodeForPackage(idClass);
			this.preferences = prefs;
		}
	}

	public Boolean getBoolean(String key, Boolean defaultValue) {
		init();
		String value = getString(key, null);
		if (value == null) {
			return defaultValue;
		} else {
			return Boolean.valueOf(value);
		}
	}

	public Integer getInt(String key, Integer defaultValue) {
		init();
		String value = getString(key, null);
		if (value == null) {
			return defaultValue;
		} else {
			return Integer.parseInt(value, 10);
		}
	}

	public Long getLong(String key, Long defaultValue) {
		init();
		String value = getString(key, null);
		if (value == null) {
			return defaultValue;
		} else {
			return Long.parseLong(value, 10);
		}
	}

	public String getString(String key, String defaultValue) {
		init();
		return this.preferences.get(key, defaultValue);
	}

	public void save() {
		init();
		try {
			this.preferences.flush();
		} catch (BackingStoreException e) {
			log.error("Cannot flush preferences", e);
		}
	}

	public void setBoolean(String key, Boolean value) {
		init();
		if (value != null) {
			this.preferences.putBoolean(key, value);
		} else {
			this.preferences.remove(key);
		}
	}

	public void setInt(String key, Integer value) {
		init();
		if (value != null) {
			this.preferences.putInt(key, value);
		} else {
			this.preferences.remove(key);
		}
	}

	public void setLong(String key, Long value) {
		init();
		if (value != null) {
			this.preferences.putLong(key, value);
		} else {
			this.preferences.remove(key);
		}
	}

	public void setString(String key, String value) {
		init();
		if (value != null) {
			this.preferences.put(key, value);
		} else {
			this.preferences.remove(key);
		}
	}

	public File getFile(String key) {
		init();
		String path = getString(key, null);
		if (path != null) {
			return new File(path);
		} else {
			return null;
		}
	}

	public void setFile(String key, File file) {
		init();
		if (file != null) {
			this.preferences.put(key, file.getAbsolutePath());
		} else {
			this.preferences.remove(key);
		}
	}

	@Override
	public void delete(String key) {
		init();
		this.preferences.remove(key);
	}

}
