package net.sf.doolin.gui.swing;

import java.awt.BorderLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;

/**
 * Panel which has a border where the user can click to collapse the panel. The
 * border remains visible and clickable to restore the panel.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class CollapsablePanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private final JPanel contentPanel;
	private final Border initialBorder;

	private boolean collapsed = false;

	private CollaspableBorder collaspableBorder;

	/**
	 * Constructor.
	 * 
	 * @param contentPanel
	 *            Content for the collapsable panel
	 * @param initialBorder
	 *            Initial border for the content
	 */
	public CollapsablePanel(JPanel contentPanel, Border initialBorder) {
		this.contentPanel = contentPanel;
		this.initialBorder = initialBorder;
		// Layout
		setLayout(new BorderLayout());
		add(contentPanel, BorderLayout.CENTER);
		// Border
		this.collaspableBorder = new CollaspableBorder(this.initialBorder);
		setBorder(this.collaspableBorder);
		// Listens for clicks on the border
		addMouseListener(new MouseAdapter() {

			@Override
			public void mouseClicked(MouseEvent e) {
				Insets i = CollapsablePanel.this.collaspableBorder
						.getBorderInsets(CollapsablePanel.this);
				if (e.getY() < i.top
						&& e.getX() > CollapsablePanel.this.getWidth() - 20) {
					setCollapsed(!isCollapsed());
					invalidate();
				}
			}

		});
	}

	/**
	 * Checks if the panel is collapsed or not
	 * 
	 * @return <code>true</code> if the panel is collapsed
	 */
	public boolean isCollapsed() {
		return this.collapsed;
	}

	/**
	 * Sets the collapsed state of the panel
	 * 
	 * @param collapsed
	 *            <code>true</code> if the panel must be collapsed
	 */
	public void setCollapsed(boolean collapsed) {
		this.collapsed = collapsed;
		this.contentPanel.setVisible(!collapsed);
		this.collaspableBorder.setCollapsed(collapsed);

		Window window = SwingUtilities.getWindowAncestor(this);
		if (window != null) {
			window.pack();
		}

		validate();
		repaint();
	}

}
