package net.sf.doolin.gui.swing;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.util.ArrayList;

import javax.swing.JSeparator;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

/**
 * {@link BorderLayout} sub-class that allows the docking of several toolbars on
 * the borders.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class DockLayout extends BorderLayout {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private ArrayList<Component> north = new ArrayList<Component>(1);
	private ArrayList<Component> south = new ArrayList<Component>(1);
	private ArrayList<Component> east = new ArrayList<Component>(1);
	private ArrayList<Component> west = new ArrayList<Component>(1);
	private Component center = null;
	private int northHeight, southHeight, eastWidth, westWidth;

	/**
	 * TOP location
	 */
	public static final int TOP = SwingConstants.TOP;
	/**
	 * BOTTOM location
	 */
	public static final int BOTTOM = SwingConstants.BOTTOM;
	/**
	 * LEFT location
	 */
	public static final int LEFT = SwingConstants.LEFT;
	/**
	 * RIGHT location
	 */
	public static final int RIGHT = SwingConstants.RIGHT;

	@Override
	public void addLayoutComponent(Component c, Object con) {
		synchronized (c.getTreeLock()) {
			if (con != null) {
				String s = con.toString();
				if (s.equals(NORTH)) {
					this.north.add(c);
				} else if (s.equals(SOUTH)) {
					this.south.add(c);
				} else if (s.equals(EAST)) {
					this.east.add(c);
				} else if (s.equals(WEST)) {
					this.west.add(c);
				} else if (s.equals(CENTER)) {
					this.center = c;
				}
			}
			c.getParent().validate();
		}
	}

	@Override
	public Dimension preferredLayoutSize(Container target) {
		synchronized (target.getTreeLock()) {
			Dimension dim = new Dimension(0, 0);

			this.northHeight = getPreferredDimension(this.north).height;
			this.southHeight = getPreferredDimension(this.south).height;
			this.eastWidth = getPreferredDimension(this.east).width;
			this.westWidth = getPreferredDimension(this.west).width;

			Dimension centerDim = this.center != null ? this.center
					.getPreferredSize() : new Dimension(0, 0);

			dim.width = this.westWidth + centerDim.width + this.eastWidth;
			dim.height = this.northHeight + centerDim.height + this.southHeight;

			return dim;
		}
	}

	@Override
	public void removeLayoutComponent(Component c) {
		this.north.remove(c);
		this.south.remove(c);
		this.east.remove(c);
		this.west.remove(c);
		if (c == this.center) {
			this.center = null;
		}
		flipSeparators(c, SwingConstants.VERTICAL);
		c.getParent().validate();
	}

	@Override
	public void layoutContainer(Container target) {
		synchronized (target.getTreeLock()) {
			Insets insets = target.getInsets();
			int top = insets.top;
			int bottom = target.getHeight() - insets.bottom;
			int left = insets.left;
			int right = target.getWidth() - insets.right;

			this.northHeight = getPreferredDimension(this.north).height;
			this.southHeight = getPreferredDimension(this.south).height;
			this.eastWidth = getPreferredDimension(this.east).width;
			this.westWidth = getPreferredDimension(this.west).width;

			placeComponents(this.north, left, top, right - left,
					this.northHeight, TOP);
			top += (this.northHeight + getVgap());

			placeComponents(this.south, left,
					bottom - this.southHeight, right - left, this.southHeight,
					BOTTOM);
			bottom -= (this.southHeight + getVgap());

			placeComponents(this.east, right - this.eastWidth, top,
					this.eastWidth, bottom - top, RIGHT);
			right -= (this.eastWidth + getHgap());

			placeComponents(this.west, left, top, this.westWidth,
					bottom - top, LEFT);
			left += (this.westWidth + getHgap());

			if (this.center != null) {
				this.center.setBounds(left, top, right - left, bottom - top);
			}
		}
	}

	// Returns the ideal width for a vertically oriented toolbar
	// and the ideal height for a horizontally oriented tollbar:
	private Dimension getPreferredDimension(ArrayList<Component> comps) {
		int w = 0, h = 0;

		for (int i = 0; i < comps.size(); i++) {
			Component c = (comps.get(i));
			Dimension d = c.getPreferredSize();
			w = Math.max(w, d.width);
			h = Math.max(h, d.height);
		}

		return new Dimension(w, h);
	}

	private void flipSeparators(Component c, int orientn) {
		if (c instanceof JToolBar
				&& UIManager.getLookAndFeel().getName().toLowerCase().indexOf(
						"windows") != -1) {
			JToolBar jtb = (JToolBar) c;
			Component comps[] = jtb.getComponents();
			if (comps != null && comps.length > 0) {
				for (int i = 0; i < comps.length; i++) {
					try {
						Component component = comps[i];
						if (component instanceof JSeparator) {
							jtb.remove(component);
							JSeparator separ = new JSeparator();
							if (orientn == SwingConstants.VERTICAL) {
								separ
										.setOrientation(SwingConstants.VERTICAL);
								separ.setMinimumSize(new Dimension(2, 6));
								separ.setPreferredSize(new Dimension(2, 6));
								separ.setMaximumSize(new Dimension(2, 100));
							} else {
								separ
										.setOrientation(SwingConstants.HORIZONTAL);
								separ.setMinimumSize(new Dimension(6, 2));
								separ.setPreferredSize(new Dimension(6, 2));
								separ.setMaximumSize(new Dimension(100, 2));
							}
							jtb.add(separ, i);
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		}
	}

	private void placeComponents(ArrayList<Component> comps,
			int x, int y, int w, int h, int orientation) {
		int offset = 0;
		Component c = null;

		if (orientation == TOP || orientation == BOTTOM) {

			offset = x;

			int totalWidth = 0;

			for (int i = 0; i < comps.size(); i++) {
				c = (comps.get(i));
				flipSeparators(c, SwingConstants.VERTICAL);
				int cwidth = c.getPreferredSize().width;
				totalWidth += cwidth;
				if (w < totalWidth && i != 0) {
					offset = x;
					if (orientation == TOP) {
						y += h;
						this.northHeight += h;
					} else if (orientation == BOTTOM) {
						this.southHeight += h;
						y -= h;
					}
					totalWidth = cwidth;
				}
				c.setBounds(x + offset, y, cwidth, h);
				offset += cwidth;
			}
			flipSeparators(c, SwingConstants.VERTICAL);
		} else {
			int totalHeight = 0;
			for (int i = 0; i < comps.size(); i++) {
				c = (comps.get(i));
				int cheight = c.getPreferredSize().height;
				totalHeight += cheight;
				if (h < totalHeight && i != 0) {
					if (orientation == LEFT) {
						x += w;
						this.westWidth += w;
					} else if (orientation == RIGHT) {
						this.eastWidth += w;
						x -= w;
					}
					totalHeight = cheight;
					offset = 0;
				}
				if (totalHeight > h) {
					cheight = h - 1;
				}
				c.setBounds(x, y + offset, w, cheight);
				offset += cheight;
			}
			flipSeparators(c, SwingConstants.HORIZONTAL);
		}
	}

}
