/*
 * Created on 9 mars 2005
 */
package net.sf.doolin.gui.swing;

import java.awt.Component;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import net.sf.doolin.gui.service.GUIPreferences;
import net.sf.doolin.gui.util.GUIStrings;
import net.sf.doolin.util.PatternFileFilter;

/**
 * This class allows the browsing of files or directories.
 * 
 * @author Damien Coraboeuf
 */
public class FileBrowser {

	private boolean modeDirectory;

	private boolean modeSave;

	private String fileFilter = "*.*";

	private String fileDescription;

	private File file;

	private final GUIPreferences preferences;

	/**
	 * Constructor with no support for preferences
	 */
	public FileBrowser() {
		this.preferences = null;
	}

	/**
	 * Constructor with support for preferences
	 * 
	 * @param preferences
	 *            Preference
	 */
	public FileBrowser(GUIPreferences preferences) {
		this.preferences = preferences;
	}

	/**
	 * Browses for a file or a directory. This method launches a file browser.
	 * 
	 * @param component
	 *            Parent component for the file browser dialog
	 * @return Selected file or directory or <code>null</code> if none has been
	 *         selected.
	 */
	public File browse(Component component) {
		JFileChooser chooser = new JFileChooser() {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			@Override
			public void approveSelection() {
				File file = getSelectedFile();
				boolean ok;
				if (file != null && FileBrowser.this.modeSave
						&& !FileBrowser.this.modeDirectory) {
					// Checks the name
					String fileName = file.getName();
					PatternFileFilter patternFileFilter = new PatternFileFilter(
							FileBrowser.this.fileFilter);
					if (!patternFileFilter.accept(fileName)) {
						fileName = patternFileFilter.format(fileName);
						file = new File(file.getParentFile(), fileName);
						setSelectedFile(file);
					}
					// Checks existency
					if (file.exists()) {
						String message = GUIStrings
								.get("FileBrowser.PromptReplaceFile", file
										.getName());
						int result = JOptionPane.showConfirmDialog(this,
								message, "", JOptionPane.YES_NO_OPTION);
						ok = result == JOptionPane.YES_OPTION;
					} else {
						ok = true;
					}
				} else {
					ok = true;
				}
				// Goes on
				if (ok) {
					super.approveSelection();
				}
			}

		};

		// Directory mode
		chooser
				.setFileSelectionMode(this.modeDirectory ? JFileChooser.DIRECTORIES_ONLY
						: JFileChooser.FILES_ONLY);
		// Filter
		if (this.modeDirectory) {
			chooser.setFileFilter(new FileFilter() {

				@Override
				public boolean accept(File f) {
					return f.isDirectory();
				}

				@Override
				public String getDescription() {
					return FileBrowser.this.fileDescription;
				}

			});
		} else {
			final PatternFileFilter patternFileFilter = new PatternFileFilter(
					this.fileFilter);
			chooser.setFileFilter(new FileFilter() {
				@Override
				public boolean accept(File f) {
					return f.isDirectory() || patternFileFilter.accept(f);
				}

				@Override
				public String getDescription() {
					return FileBrowser.this.fileDescription;
				}
			});
		}
		// Get initial file from preferences
		if (this.preferences != null && this.file == null) {
			this.file = this.preferences.getFile(FileBrowser.class.getName());
		}
		// Initial file
		if (this.file != null) {
			if (this.file.isDirectory()) {
				chooser.setCurrentDirectory(this.file);
			} else {
				chooser.setCurrentDirectory(this.file.getParentFile());
			}
		}
		// Browse
		int result;
		if (this.modeSave) {
			result = chooser.showSaveDialog(component);
		} else {
			result = chooser.showOpenDialog(component);
		}
		// Result
		if (result == JFileChooser.APPROVE_OPTION) {
			File f = chooser.getSelectedFile();
			if (f != null) {
				// Saves selection in preferences
				if (this.preferences != null) {
					this.preferences.setFile(FileBrowser.class.getName(), f);
					this.preferences.save();
				}
				this.file = f;
				return this.file;
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

	/**
	 * Returns the file description.
	 * 
	 * @return File description.
	 */
	public String getFileDescription() {
		return this.fileDescription;
	}

	/**
	 * Gets the file filter.
	 * 
	 * @return File filter.
	 * @see PatternFileFilter
	 */
	public String getFileFilter() {
		return this.fileFilter;
	}

	/**
	 * Gets if the browsing is addressing only directories.
	 * 
	 * @return <code>true</code> if only directories are browsed.
	 */
	public boolean isModeDirectory() {
		return this.modeDirectory;
	}

	/**
	 * Gets the browsing mode.
	 * 
	 * @return <code>true</code> if files/directories are to be saved,
	 *         <code>false</code> if they have to be opened.
	 */
	public boolean isModeSave() {
		return this.modeSave;
	}

	/**
	 * Set the initial file
	 * 
	 * @param f
	 *            Initial selection
	 */
	public void setFile(File f) {
		this.file = f;
	}

	/**
	 * Sets the file description.
	 * 
	 * @param fileDescription
	 *            File description
	 */
	public void setFileDescription(String fileDescription) {
		this.fileDescription = fileDescription;
	}

	/**
	 * Sets the file filter.
	 * 
	 * @param fileFilter
	 *            File filter.
	 * @see PatternFileFilter
	 */
	public void setFileFilter(String fileFilter) {
		this.fileFilter = fileFilter;
	}

	/**
	 * Sets if the browsing is addressing only directories.
	 * 
	 * @param modeDirectory
	 *            <code>true</code> if only directories are browsed
	 */
	public void setModeDirectory(boolean modeDirectory) {
		this.modeDirectory = modeDirectory;
	}

	/**
	 * Sets the browsing mode.
	 * 
	 * @param modeSave
	 *            <code>true</code> if files/directories are to be saved,
	 *            <code>false</code> if they have to be opened.
	 */
	public void setModeSave(boolean modeSave) {
		this.modeSave = modeSave;
	}

}
