package net.sf.doolin.gui.swing;

import javax.swing.Icon;

import net.sf.doolin.gui.service.IconService;
import net.sf.doolin.gui.service.IconSize;
import net.sf.doolin.gui.service.icon.DefaultIconService;
import net.sf.doolin.gui.util.GUIStrings;

import org.springframework.beans.factory.annotation.Autowired;

/**
 * This {@link LabelInfoProvider} uses a
 * {@linkplain String#format(String, Object...) format} to build the text key of
 * both the text and the icon id.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class FormattedKeyLabelInfoProvider implements LabelInfoProvider {

	private String textFormat;
	private String iconFormat;
	private IconService iconService = new DefaultIconService();

	/**
	 * Constructor, with no text and no icon.
	 */
	public FormattedKeyLabelInfoProvider() {
	}

	/**
	 * Gets the {@linkplain String#format(String, Object...) format} that is
	 * used to build the text key of the icon id.
	 * 
	 * @return Format
	 */
	public String getIconFormat() {
		return this.iconFormat;
	}

	@Override
	public LabelInfo getLabelIcon(Object item) {
		// Text
		String text = null;
		if (this.textFormat != null) {
			String textKey = String.format(this.textFormat, item);
			text = GUIStrings.get(textKey);
		}
		// Icon
		Icon icon = null;
		if (this.iconFormat != null) {
			String iconStringKey = String.format(this.iconFormat, item);
			String iconKey = GUIStrings.getIfPresent(iconStringKey);
			if (iconKey != null) {
				icon = this.iconService.getIcon(iconKey, IconSize.SMALL);
			}
		}
		// OK
		return new LabelInfo(text, icon);
	}

	/**
	 * Gets the {@linkplain String#format(String, Object...) format} that is
	 * used to get the text key of the text to display.
	 * 
	 * @return Format
	 */
	public String getTextFormat() {
		return this.textFormat;
	}

	/**
	 * Sets the {@linkplain String#format(String, Object...) format} that is
	 * used to build the text key of the icon id.
	 * 
	 * @param iconFormat
	 *            Format
	 */
	public void setIconFormat(String iconFormat) {
		this.iconFormat = iconFormat;
	}

	/**
	 * Sets the icon service used to access icons
	 * 
	 * @param iconService
	 *            Icon service
	 */
	@Autowired(required = false)
	public void setIconService(IconService iconService) {
		this.iconService = iconService;
	}

	/**
	 * Sets the {@linkplain String#format(String, Object...) format} that is
	 * used to get the text key of the text to display.
	 * 
	 * @param textFormat
	 *            Format
	 */
	public void setTextFormat(String textFormat) {
		this.textFormat = textFormat;
	}

}
