package net.sf.doolin.gui.util;

import java.awt.Color;
import java.awt.Font;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;

import net.sf.doolin.gui.service.GUIPreferences;
import net.sf.doolin.gui.spring.ColorEditor;
import net.sf.doolin.util.CodecUtils;

import org.apache.commons.lang.SerializationUtils;
import org.apache.commons.lang.StringUtils;

/**
 * Utility methods for the {@link GUIPreferences} that extends the number of
 * types that can be stored and retrieved.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class PreferencesUtils {

	private PreferencesUtils() {
	}

	/**
	 * Retrieves a {@link BigDecimal} from the preferences.
	 * 
	 * @param preferences
	 *            Preferences
	 * @param key
	 *            Key
	 * @param defaultValue
	 *            Default value if not found
	 * @return Stored {@link BigDecimal} or the default value
	 */
	public static BigDecimal getBigDecimal(GUIPreferences preferences,
			String key, BigDecimal defaultValue) {
		String value = preferences.getString(key, null);
		if (value == null) {
			return defaultValue;
		} else {
			return new BigDecimal(value);
		}
	}

	/**
	 * Retrieves a {@link Color} from the preferences.
	 * 
	 * @param preferences
	 *            Preferences
	 * @param key
	 *            Key
	 * @param defaultColor
	 *            Default value if not found
	 * @return Stored {@link Color} or the default value
	 * @see ColorEditor#parseColor(String)
	 */
	public static Color getColor(GUIPreferences preferences, String key,
			Color defaultColor) {
		String value = preferences.getString(key, null);
		if (value == null) {
			return defaultColor;
		} else {
			return ColorEditor.parseColor(value);
		}
	}

	/**
	 * Retrieves a {@link Enum} from the preferences.
	 * 
	 * @param <E>
	 *            Type of the enumeration
	 * @param preferences
	 *            Preferences
	 * @param prefKey
	 *            Key
	 * @param eClass
	 *            Class of the enumeration
	 * @param defaultValue
	 *            Default value if not found
	 * @return Stored {@link BigDecimal} or the default value
	 */
	public static <E extends Enum<E>> E getEnum(GUIPreferences preferences,
			String prefKey, Class<E> eClass, E defaultValue) {
		String value = preferences.getString(prefKey, null);
		if (value == null) {
			return defaultValue;
		} else {
			return Enum.valueOf(eClass, value);
		}
	}

	/**
	 * Retrieves a {@link Font} from the preferences.
	 * 
	 * @param preferences
	 *            Preferences
	 * @param key
	 *            Key
	 * @param defaultFont
	 *            Default value if not found
	 * @return Stored {@link Font} or the default value
	 */
	public static Font getFont(GUIPreferences preferences, String key,
			Font defaultFont) {
		String fontName = preferences.getString(key + ".family", defaultFont
				.getFamily());
		int fontStyle = preferences.getInt(key + ".style", defaultFont
				.getStyle());
		int fontSize = preferences.getInt(key + ".size", defaultFont.getSize());
		return new Font(fontName, fontStyle, fontSize);
	}

	/**
	 * Retrieves a {@link Serializable} object from the preferences.
	 * 
	 * @param <T>
	 *            Type of the {@link Serializable} object
	 * @param preferences
	 *            Preferences
	 * @param key
	 *            Key
	 * @return Stored {@link BigDecimal} or <code>null</code>
	 */
	public static <T extends Serializable> T getObject(
			GUIPreferences preferences, String key) {
		String value = preferences.getString(key, null);
		if (value != null) {
			byte[] bytes = CodecUtils.decodeBase64(value);
			@SuppressWarnings("unchecked")
			T t = (T) SerializationUtils.deserialize(bytes);
			return t;
		} else {
			return null;
		}
	}

	/**
	 * Retrieves a collection of strings from the preferences.
	 * 
	 * @param preferences
	 *            Preferences
	 * @param key
	 *            Key
	 * @return Stored collection or empty array
	 */
	public static Collection<String> getStringArray(GUIPreferences preferences,
			String key) {
		String value = preferences.getString(key, null);
		if (StringUtils.isBlank(value)) {
			return Collections.emptySet();
		} else {
			return Arrays.asList(StringUtils.split(value, ";"));
		}
	}

	/**
	 * Stores a {@link BigDecimal} in the preferences
	 * 
	 * @param preferences
	 *            Preferences
	 * @param key
	 *            Key
	 * @param value
	 *            Value to store
	 */
	public static void setBigDecimal(GUIPreferences preferences, String key,
			BigDecimal value) {
		if (value != null) {
			preferences.setString(key, value.toString());
		} else {
			preferences.delete(key);
		}
	}

	/**
	 * Stores a {@link Color} in the preferences
	 * 
	 * @param preferences
	 *            Preferences
	 * @param key
	 *            Key
	 * @param color
	 *            Value to store
	 */
	public static void setColor(GUIPreferences preferences, String key,
			Color color) {
		String value = ColorEditor.toString(color);
		preferences.setString(key, value);
	}

	/**
	 * Stores a {@link Enum} in the preferences
	 * 
	 * @param <E>
	 *            Type of the enumeration
	 * @param preferences
	 *            Preferences
	 * @param prefKey
	 *            Key
	 * @param e
	 *            Value to store
	 */
	public static <E extends Enum<E>> void setEnum(GUIPreferences preferences,
			String prefKey, Enum<E> e) {
		if (e == null) {
			preferences.delete(prefKey);
		} else {
			preferences.setString(prefKey, e.toString());
		}
	}

	/**
	 * Stores a {@link Font} in the preferences
	 * 
	 * @param preferences
	 *            Preferences
	 * @param key
	 *            Key
	 * @param font
	 *            Value to store
	 */
	public static void setFont(GUIPreferences preferences, String key, Font font) {
		preferences.setString(key + ".family", font.getFamily());
		preferences.setInt(key + ".style", font.getStyle());
		preferences.setInt(key + ".size", font.getSize());
	}

	/**
	 * Stores a {@link Serializable} object in the preferences
	 * 
	 * @param preferences
	 *            Preferences
	 * @param key
	 *            Key
	 * @param object
	 *            Value to store
	 */
	public static void setObject(GUIPreferences preferences, String key,
			Serializable object) {
		byte[] bytes = SerializationUtils.serialize(object);
		String value = CodecUtils.encodeBase64(bytes);
		preferences.setString(key, value);
	}

	/**
	 * Stores a collection of strings in the preferences
	 * 
	 * @param preferences
	 *            Preferences
	 * @param key
	 *            Key
	 * @param strings
	 *            Values to store
	 */
	public static void setStringArray(GUIPreferences preferences, String key,
			Collection<String> strings) {
		String value = StringUtils.join(strings, ";");
		preferences.setString(key, value);
	}

}
