package net.sf.doolin.gui.validation;

import java.util.Arrays;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.bus.support.SubscriberValidator;
import net.sf.doolin.gui.util.GUIStrings;
import net.sf.doolin.util.Utils;

import org.apache.commons.lang.ObjectUtils;

/**
 * Validator used for the validation of a password input.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of bean for the view
 */
public class ConfirmPasswordValidator<B> extends AbstractValidator<B> {

	private String passwordPropertyPath;
	private String confirmPasswordPropertyPath;
	private String msgKey = ValidationMsgKeys.MSG_CONFIRM_PASSWORD;
	private String labelKey;
	private String key;

	@Override
	protected void validate(GUIValidation validationResult, B bean) {
		Object passwordValue = Utils.getProperty(bean,
				this.passwordPropertyPath);
		Object confirmPasswordValue = Utils.getProperty(bean,
				this.confirmPasswordPropertyPath);
		boolean valid = true;
		if (passwordValue != null && confirmPasswordValue != null) {
			if (passwordValue.getClass().isArray()) {
				char[] p1 = (char[]) passwordValue;
				char[] p2 = (char[]) confirmPasswordValue;
				valid = Arrays.equals(p1, p2);
			} else {
				valid = ObjectUtils.equals(passwordValue, confirmPasswordValue);
			}
		} else if (passwordValue != null || confirmPasswordValue != null) {
			valid = false;
		}
		if (!valid) {
			GUIValidationMessage message = new GUIValidationMessage(this.key,
					this.msgKey, GUIStrings.get(this.labelKey));
			validationResult.addMessage(message);
		}
	}

	@Override
	public void subscribe(SubscriberValidator validator, B bean,
			Runnable runnable) {
		PropertyChangeSupport.subscribe(validator, bean,
				this.passwordPropertyPath, runnable);
		PropertyChangeSupport.subscribe(validator, bean,
				this.confirmPasswordPropertyPath, runnable);
	}

	/**
	 * Path to the property that contains the password
	 * 
	 * @return Property path
	 */
	public String getPasswordPropertyPath() {
		return this.passwordPropertyPath;
	}

	/**
	 * Path to the property that contains the password
	 * 
	 * @param passwordPropertyPath
	 *            Property path
	 */
	public void setPasswordPropertyPath(String passwordPropertyPath) {
		this.passwordPropertyPath = passwordPropertyPath;
	}

	/**
	 * Path to the property that contains the confirmation of the password
	 * 
	 * @return Property path
	 */
	public String getConfirmPasswordPropertyPath() {
		return this.confirmPasswordPropertyPath;
	}

	/**
	 * Path to the property that contains the confirmation of the password
	 * 
	 * @param confirmPasswordPropertyPath
	 *            Property path
	 */
	public void setConfirmPasswordPropertyPath(
			String confirmPasswordPropertyPath) {
		this.confirmPasswordPropertyPath = confirmPasswordPropertyPath;
	}

	/**
	 * Key for the text that defines the password field
	 * 
	 * @return Text key
	 */
	public String getLabelKey() {
		return this.labelKey;
	}

	/**
	 * Key for the text that defines the password field
	 * 
	 * @param labelKey
	 *            Text key
	 */
	public void setLabelKey(String labelKey) {
		this.labelKey = labelKey;
	}

	/**
	 * Key that identifies the password field
	 * 
	 * @return Field key
	 */
	public String getKey() {
		return this.key;
	}

	/**
	 * Key that identifies the password field
	 * 
	 * @param key
	 *            Field key
	 */
	public void setKey(String key) {
		this.key = key;
	}

	/**
	 * Key for the text that defines the validation message
	 * 
	 * @return Text key
	 */
	public String getMsgKey() {
		return this.msgKey;
	}

	/**
	 * Key for the text that defines the validation message. It is set by
	 * default to {@link ValidationMsgKeys#MSG_CONFIRM_PASSWORD}.
	 * 
	 * @param msgKey
	 *            Text key
	 */
	public void setMsgKey(String msgKey) {
		this.msgKey = msgKey;
	}

}
