package net.sf.doolin.gui.validation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.bus.support.SubscriberValidator;
import net.sf.doolin.gui.util.GUIStrings;
import net.sf.doolin.gui.validation.support.FalseObjectValidator;
import net.sf.doolin.gui.validation.support.NotBlankObjectValidator;
import net.sf.doolin.gui.validation.support.NotEmptyObjectValidator;
import net.sf.doolin.gui.validation.support.NotNullObjectValidator;
import net.sf.doolin.gui.validation.support.ObjectValidator;
import net.sf.doolin.gui.validation.support.TrueObjectValidator;
import net.sf.doolin.util.Utils;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Required;

/**
 * Validator based on textual declarations.
 * <p>
 * Each validation has the following format:
 * 
 * <pre>
 * &lt;validation&gt; := &lt;context&gt;=&lt;validators&gt;
 * 
 * &lt;context&gt; := &lt;key&gt;:&lt;property&gt;[:&lt;name&gt;]
 * &lt;validators&gt; := Comma-separated list of &lt;validator&gt;
 * </pre>
 * 
 * where:
 * <ul>
 * <li><code>key</code>: Key that identifies the field in error, usually mapped
 * to the {@linkplain net.sf.doolin.gui.field.FieldDescriptor#getName() field
 * name}
 * <li><code>property</code>: Path to the property to test on the model
 * <li><code>name</code>: String key to the name of the field (defaults to the
 * {@linkplain net.sf.doolin.gui.field.FieldDescriptor#getName() field name})
 * <li><code>validator</code>: key used to identify an {@link ObjectValidator}
 * for the validation (see below)
 * </ul>
 * 
 * <code>validator</code> keys are used to identify {@link ObjectValidator}
 * instances. Predefined ones are:
 * <ul>
 * <li>{@link NotNullObjectValidator notNull}
 * <li>{@link NotBlankObjectValidator notBlank}
 * </ul>
 * 
 * It is possible to declare new keys using the
 * {@link #setObjectValidators(Map)} method or the <code>objectValidators</code>
 * property.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of object for the view
 */
public class SimpleValidator<B> extends AbstractValidator<B> {

	/**
	 * Validation support that operates as a link to an {@link ObjectValidator}
	 * instance.
	 * 
	 * @author Damien Coraboeuf
	 * 
	 */
	protected class SimpleValidation {

		private final String property;
		private final ObjectValidator objectValidator;
		private final String key;
		private final String name;

		/**
		 * Instantiates a new simple validation.
		 * 
		 * @param property
		 *            the property
		 * @param objectValidator
		 *            the object validator
		 * @param key
		 *            the key
		 * @param name
		 *            the name
		 */
		public SimpleValidation(String property,
				ObjectValidator objectValidator, String key, String name) {
			this.property = property;
			this.objectValidator = objectValidator;
			this.key = key;
			this.name = name;
		}

		/**
		 * Subscribes the validation to the model changes.
		 * 
		 * @param validator
		 *            Subscription validator
		 * @param model
		 *            Model to subscribe to
		 * @param runnable
		 *            Task to subscribe
		 * @see PropertyChangeSupport#subscribe(SubscriberValidator, Object,
		 *      String, Runnable)
		 */
		public void subscribe(SubscriberValidator validator, B model,
				Runnable runnable) {
			PropertyChangeSupport.subscribe(validator, model, this.property,
					runnable);
		}

		/**
		 * Validate.
		 * 
		 * @param result
		 *            the result
		 * @param bean
		 *            the bean
		 */
		public void validate(GUIValidation result, B bean) {
			Object propertyValue = Utils.getProperty(bean, this.property);
			boolean ok = this.objectValidator.validate(propertyValue);
			if (!ok) {
				GUIValidationMessage message = new GUIValidationMessage(
						this.key, this.objectValidator.getMsgKey(),
						this.objectValidator.getParams(GUIStrings
								.get(this.name)));
				result.addMessage(message);
			}
		}

	}

	private List<String> validations;

	private Map<String, ObjectValidator> objectValidators = new HashMap<String, ObjectValidator>();

	private Map<String, ObjectValidator> defaultObjectValidators = new HashMap<String, ObjectValidator>();

	private List<SimpleValidation> validationList = new ArrayList<SimpleValidation>();

	private ObjectValidator getObjectValidator(String name) {
		ObjectValidator o = this.objectValidators.get(name);
		if (o != null) {
			return o;
		} else {
			o = this.defaultObjectValidators.get(name);
			if (o != null) {
				return o;
			} else {
				throw new IllegalArgumentException(
						"Cannot find object validator with name " + name);
			}
		}
	}

	/**
	 * Gets the map between the <code>validator</code> keys and the
	 * {@link ObjectValidator} instances.
	 * 
	 * @return Validators
	 */
	public Map<String, ObjectValidator> getObjectValidators() {
		return this.objectValidators;
	}

	/**
	 * Gets the validations.
	 * 
	 * @return the validations
	 */
	public List<String> getValidations() {
		return this.validations;
	}

	/**
	 * Initialises the object validators.
	 * 
	 * @throws IllegalArgumentException
	 *             If there is problem of parsing
	 */
	@PostConstruct
	public void init() throws IllegalArgumentException {
		// Fills default validators
		this.defaultObjectValidators.put("notNull",
				new NotNullObjectValidator());
		this.defaultObjectValidators.put("notBlank",
				new NotBlankObjectValidator());
		this.defaultObjectValidators.put("notEmpty",
				new NotEmptyObjectValidator());
		this.defaultObjectValidators.put("true", new TrueObjectValidator());
		this.defaultObjectValidators.put("false", new FalseObjectValidator());
		// Each line of validation
		for (String validationLine : this.validations) {
			validationLine = StringUtils.trim(validationLine);
			String firstPart = StringUtils.substringBefore(validationLine, "=");
			String[] defTokens = StringUtils.split(firstPart, ":");
			if (defTokens.length < 2 || defTokens.length > 3) {
				throw new IllegalArgumentException(
						"Cannot parse the name part of " + validationLine);
			}
			String key = defTokens[0];
			String property = defTokens[1];
			String name;
			if (defTokens.length > 2) {
				name = defTokens[2];
			} else {
				name = key;
			}
			String secondPart = StringUtils.substringAfter(validationLine, "=");
			String[] valTokens = StringUtils.split(secondPart, ",");
			for (String valToken : valTokens) {
				// Existing validator
				ObjectValidator objectValidator = getObjectValidator(valToken);
				// Creates the validation
				SimpleValidation validation = new SimpleValidation(property,
						objectValidator, key, name);
				// Adds it to the list
				this.validationList.add(validation);
			}
		}
	}

	/**
	 * Sets the map between the <code>validator</code> keys and the
	 * {@link ObjectValidator} instances.
	 * 
	 * @param objectValidators
	 *            Validators
	 */
	public void setObjectValidators(
			Map<String, ObjectValidator> objectValidators) {
		this.objectValidators = objectValidators;
	}

	/**
	 * Sets the validations.
	 * 
	 * @param validations
	 *            the new validations
	 */
	@Required
	public void setValidations(List<String> validations) {
		this.validations = validations;
	}

	public void subscribe(
			net.sf.doolin.bus.support.SubscriberValidator validator, B model,
			Runnable runnable) {
		for (SimpleValidation validation : this.validationList) {
			validation.subscribe(validator, model, runnable);
		}
	}

	/**
	 * {@inheritDoc}.
	 * 
	 * @param validationResult
	 *            the validation result
	 * @param bean
	 *            the bean
	 */
	@Override
	protected void validate(GUIValidation validationResult, B bean) {
		for (SimpleValidation validation : this.validationList) {
			validation.validate(validationResult, bean);
		}
	}

}
