package net.sf.doolin.gui.view.decorator;

import static net.sf.doolin.gui.view.decorator.DecorationStyle.BORDER_STYLE_BORDER_ONLY;
import static net.sf.doolin.gui.view.decorator.DecorationStyle.BORDER_STYLE_BORDER_TITLE;
import static net.sf.doolin.gui.view.decorator.DecorationStyle.BORDER_STYLE_TITLE_BAR;
import static net.sf.doolin.gui.view.decorator.DecorationStyle.PARAM_BORDER_STYLE;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.SystemColor;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.service.IconSize;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.support.GUIViewUtils;

import org.apache.commons.lang.StringUtils;

/**
 * Default decorator that decorates the view with a title, according to the
 * {@linkplain DecorationStyle decoration style}.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class TitledGUIViewDecorator extends AbstractGUIViewDecorator {

	private static final int TITLE_PADDING = 2;
	private static final int TITLE_GAP = 2;

	@Override
	public <V> JComponent decorate(GUIView<V> view, JComponent component,
			DecorationStyle decorationStyle) {
		// Border style
		String borderStyle = BORDER_STYLE_TITLE_BAR;
		if (decorationStyle != null
				&& decorationStyle.isParameterDefined(PARAM_BORDER_STYLE)) {
			borderStyle = (String) decorationStyle
					.getParamater(PARAM_BORDER_STYLE);
		}
		// Title bar
		if (BORDER_STYLE_TITLE_BAR.equals(borderStyle)) {
			return decorateWithTitleBar(view, component);
		}
		// Border only
		else if (BORDER_STYLE_BORDER_ONLY.equals(borderStyle)) {
			component.setBorder(BorderFactory.createEtchedBorder());
			return component;
		}
		// Border and title
		else if (BORDER_STYLE_BORDER_TITLE.equals(borderStyle)) {
			return decorateWithTitledBorder(view, component);
		}
		// No decoration
		else {
			return component;
		}
	}

	/**
	 * Decorates the component with a title bar
	 * 
	 * @param <V>
	 *            Type of view data
	 * @param view
	 *            View
	 * @param component
	 *            View component to decorate
	 * @return Decorated component
	 */
	protected <V> JComponent decorateWithTitleBar(GUIView<V> view,
			JComponent component) {
		// Panel
		JPanel panel = new JPanel(new BorderLayout(0, TITLE_GAP));
		// Title panel
		JPanel panelTitle = new JPanel(new FlowLayout(FlowLayout.LEADING,
				TITLE_PADDING, TITLE_PADDING));
		panelTitle.setBorder(BorderFactory.createRaisedBevelBorder());
		panelTitle.setBackground(SystemColor.activeCaption);
		panel.add(panelTitle, BorderLayout.PAGE_START);
		// Title label
		final GUIExpression titleGUIExpression = GUIViewUtils
				.getTitleExpression(view);
		final JLabel labelTitle = new JLabel(titleGUIExpression.getValue());
		labelTitle.setForeground(SystemColor.activeCaptionText);
		panelTitle.add(labelTitle);
		titleGUIExpression.connect(null, labelTitle, "text");
		// Icon
		String iconId = view.getViewDescriptor().getIconId();
		if (StringUtils.isNotBlank(iconId)) {
			Icon icon = getIconService().getIcon(iconId, IconSize.SMALL);
			labelTitle.setIcon(icon);
		}
		// Main content
		panel.add(component, BorderLayout.CENTER);
		// OK
		return panel;
	}

	/**
	 * Decorates the component with a titled border.
	 * 
	 * @param <V>
	 *            Type of view data
	 * @param view
	 *            View
	 * @param component
	 *            View component to decorate
	 * @return Decorated component
	 */
	protected <V> JComponent decorateWithTitledBorder(GUIView<V> view,
			JComponent component) {
		// Panel
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(component, BorderLayout.CENTER);
		// Main border
		Border etchedBorder = BorderFactory.createEtchedBorder();
		// Title border
		final TitledBorder titledBorder = new TitledBorder(etchedBorder);
		// Title label
		final GUIExpression titleGUIExpression = GUIViewUtils
				.getTitleExpression(view);
		titledBorder.setTitle(titleGUIExpression.getValue());
		titleGUIExpression.connect(null, titledBorder, "title");
		// Sets the border
		panel.setBorder(titledBorder);
		// OK
		return panel;
	}
}
