package net.sf.doolin.gui.view.descriptor;

import net.sf.doolin.gui.action.swing.ActionFactory;
import net.sf.doolin.gui.validation.GUIValidator;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.decorator.GUIViewDecorator;
import net.sf.doolin.gui.window.GUIWindow;

/**
 * Descriptor for a view.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of the view data.
 */
public interface GUIViewDescriptor<V> {

	/**
	 * Closes the view
	 * 
	 * @param view
	 *            View to close
	 * @param force
	 *            <code>true</code> if the view <i>must</i> be closed.
	 * @return <code>true</code> if the view could be closed
	 */
	boolean close(GUIView<V> view, boolean force);

	/**
	 * Creates a view according to this descriptor
	 * 
	 * @param parentWindow
	 *            Parent window
	 * @param model
	 *            Model for the view
	 * @return View
	 */
	GUIView<V> createView(GUIWindow<?> parentWindow, V model);

	/**
	 * Returns the action factory
	 * 
	 * @return Action factory
	 */
	ActionFactory getActionFactory();

	/**
	 * Gets the icon id
	 * 
	 * @return Icon id
	 */
	String getIconId();

	/**
	 * Gets the ID for the view (can depend on the model)
	 * 
	 * @param model
	 *            Model for the view
	 * @return Computed id
	 */
	String getID(V model);

	/**
	 * Returns the expression used to compute the title of any view
	 * 
	 * @return Expression
	 */
	String getTitleExpression();

	/**
	 * Gets the validator to be used for the view
	 * 
	 * @return Validator
	 */
	GUIValidator<V> getValidator();

	/**
	 * Additional decorator for the view
	 * 
	 * @return Decorator or <code>null</code> if none is defined.
	 */
	GUIViewDecorator getViewDecorator();

	/**
	 * Returns the view ID expression
	 * 
	 * @return Expression for the view ID
	 */
	String getViewId();

	/**
	 * This method is called after the view is ready to be displayed, giving a
	 * last chance to the descriptor to perform some last-minute initialization.
	 * 
	 * @param view
	 *            View to initialize
	 */
	void initState(GUIView<V> view);

}
