package net.sf.doolin.gui.window.descriptor;

import java.util.List;

import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.action.swing.ActionFactory;
import net.sf.doolin.gui.app.GUIApplication;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.descriptor.GUIViewDescriptor;
import net.sf.doolin.gui.window.GUIWindow;
import net.sf.doolin.gui.window.GUIWindowListener;
import net.sf.doolin.gui.window.support.WindowSize;

/**
 * Window descriptor.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of data for the window
 */
public interface GUIWindowDescriptor<B> {

	/**
	 * Activates the view in the window.
	 * 
	 * @param window
	 *            Window
	 * @param view
	 *            View
	 */
	void activateView(GUIWindow<B> window, GUIView<?> view);

	/**
	 * Creates a new window.
	 * 
	 * @param application
	 *            Application owner
	 * @param windowData
	 *            Data for the window
	 * @return Opened window (not displayed yet)
	 */
	GUIWindow<B> createWindow(GUIApplication application, B windowData);

	/**
	 * Gets the action factory.
	 * 
	 * @return the action factory
	 */
	ActionFactory getActionFactory();

	/**
	 * Returns the {@linkplain GUIAction action} to call when the window is
	 * about to be closed
	 * 
	 * @return Action to call
	 */
	GUIAction getCloseAction();

	/**
	 * Returns the size specification
	 * 
	 * @return Size
	 */
	WindowSize getSize();

	/**
	 * Gets an already opened window for the given model.
	 * 
	 * The window is identified among the other ones using an ID generated using
	 * the <code>id</code> expression of the window descriptor.
	 * 
	 * @param application
	 *            Application owner
	 * @param windowData
	 *            Data for the window
	 * @return Opened window if existing, <code>null</code> otherwise.
	 */
	GUIWindow<B> getWindow(GUIApplication application, B windowData);

	/**
	 * Returns the ID of a window model.
	 * 
	 * @param windowData
	 *            Data for the window
	 * @return ID
	 */
	String getWindowId(B windowData);

	List<GUIWindowListener<B>> getWindowListeners();

	/**
	 * Hides the view in the window.
	 * 
	 * @param window
	 *            Window
	 * @param view
	 *            View
	 */
	void hideView(GUIWindow<B> window, GUIView<?> view);

	/**
	 * Action to execute when a window is closed.
	 * 
	 * @param window
	 *            Closed window
	 */
	void onCloseWindow(GUIWindow<B> window);

	/**
	 * Action to execute when a window is about to be closed
	 * 
	 * @param window
	 *            Window to close
	 */
	void onClosingWindow(GUIWindow<B> window);

	/**
	 * This method is called when the window is displayed for the very first
	 * time
	 * 
	 * @param window
	 *            Window which is opened for the very first time
	 */
	void onOpenedWindow(GUIWindow<B> window);

	/**
	 * Opens a view in the window
	 * 
	 * @param parentWindow
	 *            Parent window
	 * @param viewDescriptor
	 *            Descriptor for the view to open
	 * @param viewData
	 *            Data for the view
	 * @param <V>
	 *            Type of data for the view
	 * @return Opened view
	 */
	<V> GUIView<V> openView(GUIWindow<B> parentWindow,
			GUIViewDescriptor<V> viewDescriptor, V viewData);

}
