package net.sf.doolin.gui.window.support;

import java.awt.Window;
import java.awt.event.WindowListener;

import javax.swing.Icon;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.swing.SwingUtils;

/**
 * Abstract {@link IWindow} implementation that holds the concrete
 * implementation of a {@link Window}.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <W>
 *            Type of {@link Window} to hold
 */
public abstract class AbstractIWindow<W extends Window> implements IWindow {

	private final ActionContext actionContext;
	private final W window;

	/**
	 * Constructor
	 * 
	 * @param actionContext
	 */
	public AbstractIWindow(ActionContext actionContext) {
		this.actionContext = actionContext;
		this.window = buildWindow();
	}

	/**
	 * @see Window#addWindowListener(WindowListener)
	 */
	@Override
	public void addWindowListener(WindowListener windowListener) {
		this.window.addWindowListener(windowListener);
	}

	/**
	 * Actually builds the window instance
	 * 
	 * @return Window
	 */
	protected abstract W buildWindow();

	/**
	 * @see Window#dispose()
	 */
	@Override
	public void dispose() {
		this.window.dispose();
	}

	/**
	 * Gets the associated action context
	 * 
	 * @return Action context
	 */
	protected ActionContext getActionContext() {
		return this.actionContext;
	}

	@Override
	public int getHeight() {
		return this.window.getHeight();
	}

	@Override
	public int getWidth() {
		return this.window.getWidth();
	}

	/**
	 * Gets the actual window instance
	 * 
	 * @return Window
	 */
	protected W getWindow() {
		return this.window;
	}

	/**
	 * @see Window#setIconImage(java.awt.Image)
	 * @see SwingUtils#toImage(Icon)
	 */
	@Override
	public void setIcon(Icon icon) {
		this.window.setIconImage(SwingUtils.toImage(icon));
	}

	/**
	 * @see Window#setLocationRelativeTo(java.awt.Component)
	 * @see Window#getOwner()
	 */
	@Override
	public void setLocation() {
		this.window.setLocationRelativeTo(this.window.getOwner());
	}

	/**
	 * @see Window#setSize(int, int)
	 */
	@Override
	public void setSize(int width, int height) {
		this.window.setSize(width, height);
	}

	/**
	 * @see Window#setVisible(boolean)
	 */
	@Override
	public void show() {
		this.window.setVisible(true);
	}

}
