package net.sf.doolin.gui.wizard.rule.condition;

import java.util.List;
import java.util.Map;

import net.sf.doolin.gui.wizard.rule.AbstractWizardRules;

import org.apache.commons.lang.ObjectUtils;
import org.springframework.beans.factory.annotation.Required;

/**
 * Wizard rule based on a list of conditions. Each view is associated with a
 * list of individual {@linkplain WizardRule wizard rules} to be evaluated.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of bean for the wizard
 */
public class ConditionWizardRules<B> extends AbstractWizardRules<B> {

	private Map<String, List<WizardRule>> rules;

	/**
	 * Checks if the wizard can be finished. Each available rule for the current
	 * view is evaluated and if at least one of them returns
	 * {@link WizardRule#FINISH}, the wizard can be finished.
	 */
	@Override
	public boolean canFinish(String currentViewId, B model) {
		// Gets the list of rules
		List<WizardRule> ruleList = this.rules.get(currentViewId);
		if (ruleList == null || ruleList.isEmpty()) {
			return true;
		} else {
			for (WizardRule rule : ruleList) {
				String result = rule.getResult();
				if (ObjectUtils.equals(WizardRule.FINISH, result)) {
					boolean ok = rule.evaluate(model);
					if (ok) {
						return true;
					}
				}
			}
			return false;
		}
	}

	/**
	 * Checks if the wizard can be navigated to the next view. The next view is
	 * the result of the first available rule that is evaluated to
	 * <code>true</code> .
	 */
	@Override
	public String getNext(String currentViewId, B model) {
		// Gets the list of rules
		List<WizardRule> ruleList = this.rules.get(currentViewId);
		if (ruleList == null || ruleList.isEmpty()) {
			return null;
		} else {
			for (WizardRule rule : ruleList) {
				boolean ok = rule.evaluate(model);
				if (ok) {
					return rule.getResult();
				}
			}
		}
		return null;
	}

	/**
	 * Sets the list of rules, indexed by view names.
	 * 
	 * @param rules
	 *            Map of rules per view name
	 */
	@Required
	public void setRules(Map<String, List<WizardRule>> rules) {
		this.rules = rules;
	}

}
