package net.sf.doolin.gui.wizard.state;

import java.util.Stack;

import net.sf.doolin.bus.bean.Bean;
import net.sf.doolin.bus.bean.NotificationMode;
import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.wizard.rule.WizardRules;

/**
 * Definition of the current state of a wizard.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of bean for the wizard
 */
public class WizardState<B> extends Bean {

	private static final long serialVersionUID = 1L;

	/**
	 * Property name for the current view: {@value}
	 */
	public static final String CURRENT_VIEW = "currentView";

	private final B wizardModel;
	private final WizardRules<B> wizardRules;
	private final Stack<String> viewStack;

	private String currentView;

	/**
	 * Constructor.
	 * 
	 * @param model
	 *            Wizard bean
	 * @param wizardRules
	 *            Wizard navigation rules
	 * @param initialView
	 *            Name of the initial view
	 */
	public WizardState(B model, WizardRules<B> wizardRules, String initialView) {
		this.wizardModel = model;
		this.viewStack = new Stack<String>();
		this.viewStack.push(initialView);
		this.currentView = initialView;
		this.wizardRules = wizardRules;
	}

	/**
	 * Checks if the wizard can be finished
	 * 
	 * @return Finish-enabled status
	 * @see WizardRules#canFinish(String, Object)
	 */
	public boolean canFinish() {
		return this.wizardRules.canFinish(this.currentView, this.wizardModel);
	}

	/**
	 * Checks if the wizard can be navigated to the next view. The
	 * {@link WizardRules#getNext(String, Object)} is called and "Next" is
	 * enabled if the returned view is different than <code>null</code>.
	 * 
	 * @return Next-enabled status
	 * @see WizardRules#getNext(String, Object)
	 */
	public boolean canNext() {
		return this.wizardRules.getNext(this.currentView, this.wizardModel) != null;
	}

	/**
	 * Checks if the wizard can be navigated to the previous view. It is
	 * possible if the stack of navigated views if not empty.
	 * 
	 * @return Previous-enabled status
	 */
	public boolean canPrevious() {
		return this.viewStack.size() > 1;
	}

	/**
	 * Actually navigates to the next view in the wizard. Before the navigation
	 * is actually done, any
	 * {@linkplain WizardRules#getTransitionAction(String) transaction action}
	 * for the next view is executed. Then, the next view becomes the current
	 * view and this change is notified on the bus.
	 * 
	 * @param actionContext
	 *            Action context
	 * @see WizardRules#getNext(String, Object)
	 * @see #setCurrentView(String)
	 */
	public void doNext(ActionContext actionContext) {
		String next = this.wizardRules.getNext(this.currentView,
				this.wizardModel);
		if (next != null) {
			// Executes any transition action
			GUIAction action = this.wizardRules.getTransitionAction(next);
			if (action != null) {
				boolean result = action.execute(actionContext);
				if (!result) {
					return;
				}
			}
			// Switches to the next view
			NotificationMode mode = bufferedNotification();
			try {
				this.viewStack.push(next);
				setCurrentView(next);
			} finally {
				setNotificationMode(mode);
			}
		}
	}

	/**
	 * Actually navigates to the previous view. The current view name is popped
	 * for the stack of navigated views and the new stack top becomes the
	 * current view. No action is executed.
	 * 
	 * @param actionContext
	 *            Action context
	 * @see #setCurrentView(String)
	 */
	public void doPrevious(ActionContext actionContext) {
		if (this.viewStack.size() > 1) {
			NotificationMode mode = bufferedNotification();
			try {
				String viewId = this.viewStack.pop();
				assert viewId.equals(this.currentView);
				viewId = this.viewStack.peek();
				setCurrentView(viewId);
			} finally {
				setNotificationMode(mode);
			}
		}
	}

	/**
	 * Gets the current view
	 * 
	 * @return Current view name
	 */
	public String getCurrentView() {
		return this.currentView;
	}

	/**
	 * Gets the bean for the wizard
	 * 
	 * @return Bean
	 */
	public B getWizardModel() {
		return this.wizardModel;
	}

	/**
	 * Navigates to the given view. The new name is notified on the bus.
	 * 
	 * @param currentView
	 *            View name
	 */
	public void setCurrentView(String currentView) {
		notify(CURRENT_VIEW, this.currentView, this.currentView = currentView);
	}

}
