package net.sf.doolin.gui.docking;

import java.util.Map;

import net.sf.doolin.util.Utils;

import org.apache.commons.lang.StringUtils;

import com.javadocking.dock.Position;

/**
 * Parser for a {@linkplain DockingConstraint#setDockingPath(String) docking
 * path}.
 * <p>
 * The format of a docking path is a semi-colon-separated list of paths, each of
 * them having the following format:
 * 
 * <pre>
 * &lt;path&gt; := &lt;position&gt;{'?'&lt;query&gt;}
 * &lt;query&gt; := param=value&amp;param=value&amp;...
 * </pre>
 * 
 * <p>
 * Position can be <code>TOP</code>, <code>BOTTOM</code>, <code>LEFT</code> or
 * <code>RIGHT</code>.
 * 
 * <p>
 * Parameters can be:
 * <ul>
 * <li><code>size</code> - size of pixels of the dock to create
 * <li><code>type</code> - type of the dock to create (see {@link DockType}).
 * </ul>
 * 
 * @author Damien Coraboeuf
 * 
 */
public class DockingPathConstraints {

	private static final int DEFAULT_SIZE = 150;

	private static final String TYPE = "type";

	private static final String SIZE = "size";

	private static final String QUERY_SEPARATOR = "?";

	private final String path;

	private int size;
	private DockType type;
	private int position;

	/**
	 * Instantiates a new docking path constraints.
	 * 
	 * @param path
	 *            the path
	 */
	public DockingPathConstraints(String path) {
		this.path = path;
		parse();
	}

	/**
	 * Parses the path.
	 */
	protected void parse() {
		String positionString = StringUtils.substringBefore(path,
				QUERY_SEPARATOR);
		String query = StringUtils.substringAfter(path, QUERY_SEPARATOR);
		// Position
		position = (Integer) Utils.getConstant(Position.class, positionString);
		// Parses the query
		Map<String, String> params = Utils.parseQuery(query);
		// Size
		String sizeString = params.get(SIZE);
		if (StringUtils.isNotBlank(sizeString)) {
			size = Integer.parseInt(sizeString, 10);
		} else {
			size = DEFAULT_SIZE;
		}
		// Type
		String typeString = params.get(TYPE);
		if (StringUtils.isNotBlank(typeString)) {
			typeString = StringUtils.upperCase(typeString);
			type = DockType.valueOf(typeString);
		} else {
			type = DockType.SPLIT;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return path;
	}

	/**
	 * Gets the path.
	 * 
	 * @return the path
	 */
	public String getPath() {
		return path;
	}

	/**
	 * Gets the size.
	 * 
	 * @return the size
	 */
	public int getSize() {
		return size;
	}

	/**
	 * Gets the type.
	 * 
	 * @return the type
	 */
	public DockType getType() {
		return type;
	}

	/**
	 * Gets the position.
	 * 
	 * @return the position
	 */
	public int getPosition() {
		return position;
	}

}
